# -*- coding: utf-8 -*-

class Dracurouge < DiceBot
  def initialize
    super
    @d66Type = 1
  end

  def gameName
    'ドラクルージュ'
  end

  def gameType
    "Dracurouge"
  end

  def getHelpMessage
    return <<MESSAGETEXT
・行い判定（DRx+y）
　x：振るサイコロの数（省略時４）、y：渇き修正（省略時０）
　例） DR　DR6　DR+1　DR5+2
・抗い判定（DRRx）
　x：振るサイコロの数
　例） DRR3
・原風景表（ST）、叙勲表（CO）、叙勲後表（CA）、遥か過去表（EP）
　原罪表（OS）、受難表（PN）、近況表（RS）、平和な過去表（PP）
・堕落表（CTx） x：渇き （例） CT3
・堕落の兆し表（CS）、拡張・堕落の兆し表（ECS）
・絆内容決定表（BT）
・反応表（RTxy）x：血統、y：道　xy省略で一括表示
　血統　D：ドラク、R：ローゼンブルク、H：ヘルズガルド、M：ダストハイム
　　　　A：アヴァローム　N：ノスフェラス、G：ゲイズヴァルト、K：カインシルト
　道　F：領主、G：近衛、R：遍歴、W：賢者、J：狩人、N：夜獣
　　　E：将軍、B：僧正、H：空駆、K：船長、L：寵童、V：仲立、U：技師、D：博士
　　　S：星読、G2：後見
　例）RT（一括表示）、RTDF（ドラクの領主）、RTAG2（アヴァロームの後見人）
・異端の反応表（HRTxy）x：血統、y：道　xy省略で一括表示
　血統　L：異端卿、V：ヴルコラク、N：ナハツェーラ、K：カルンシュタイン
　　　　G：グリマルキン、S：ストリガ、M：メリュジーヌ、F：フォーン
　　　　H：ホムンクルス、E：エナメルム、S2：サングィナリエ、A：アールヴ
　　　　V2：ヴィーヴル、L2：ルーガルー、A2：アルラウネ、F2：フリッガ
　道　W：野伏、N：流浪、S：密使、H：魔女、F：剣士、X：検体
　例）HRT（一括表示）、HRTVW（ヴルコラクの野伏）、HRTF2X（フリッガの検体）
・D66ダイスあり
MESSAGETEXT
  end

  def rollDiceCommand(command)
    debug('rollDiceCommand')

    # get～DiceCommandResultという名前のメソッドを集めて実行、
    # 結果がnil以外の場合それを返して終了。
    return analyzeDiceCommandResultMethod(command)
  end

  def getConductDiceCommandResult(command)
    return nil unless /^DR(\d*)(\+(\d+))?$/ === command

    diceCount = $1.to_i
    diceCount = 4 if diceCount == 0
    thirstyPoint = $3.to_i

    diceList = rollDiceList(diceCount)

    gloryDiceCount = getGloryDiceCount(diceList)
    gloryDiceCount.times{ diceList << 10 }

    diceList, calculationProcess = getThirstyAddedResult(diceList, thirstyPoint)
    thirstyPointMarker = (thirstyPoint == 0 ? "" : "+#{thirstyPoint}")

    result = "(#{command}) ＞ #{diceCount}D6#{thirstyPointMarker} ＞ "
    result += "[ #{calculationProcess} ] ＞ " unless calculationProcess.empty?
    result += "[ #{diceList.join(', ')} ]"
    return result
  end

  def rollDiceList(diceCount)
    _, str = roll(diceCount, 6)
    diceList = str.split(/,/).collect{|i|i.to_i}.sort

    return diceList
  end

  def getGloryDiceCount(diceList)
    oneCount = countTargetDice(diceList, 1)
    sixCount = countTargetDice(diceList, 6)

    gloryDiceCount = (oneCount / 2) + (sixCount / 2)
    return gloryDiceCount
  end

  def countTargetDice(diceList, target)
    diceList.select{|i|i == target}.count
  end

  def getThirstyAddedResult(diceList, thirstyPoint)
    return diceList, '' if thirstyPoint == 0

    targetIndex = diceList.rindex{|i| i <= 6}
    return diceList, '' if targetIndex.nil?

    textList = []

    diceList.each_with_index do |item, index|
      if targetIndex == index
        textList << "#{item}+#{thirstyPoint}"
      else
        textList << item.to_s
      end
    end

    diceList[targetIndex] += thirstyPoint

    return diceList, textList.join(', ')
  end

  def getResistDiceCommandResult(command)
    return nil unless /^DRR(\d+)$/ === command

    diceCount = $1.to_i
    diceCount = 4 if diceCount == 0

    diceList = rollDiceList(diceCount)

    result = "(#{command}) ＞ #{diceCount}D6 ＞ [ #{diceList.join(', ')} ]"
    return result
  end

  def getReactionDiceCommandResult(command)
    return nil unless /^RT((\w\d*)(\w\d*))?/ === command
    typeText1 = $2
    typeText2 = $3

    name = "反応表"

    return getReactionText(name, typeText1, typeText2, getReactionInfoBlood, getReactionInfoStyle)
  end

  def getHeresyReactionDiceCommandResult(command)
    return nil unless /^HRT((\w\d*)(\w\d*))?/ === command
    typeText1 = $2
    typeText2 = $3

    name = "異端の反応表"

    return getReactionText(name, typeText1, typeText2, getHeresyReactionInfoBlood, getHeresyReactionInfoStyle)
  end

  def getReactionText(name, typeText1, typeText2, infos1, infos2)

    return nil unless checkTypeText(typeText1, infos1)
    return nil unless checkTypeText(typeText2, infos2)

    ten_value, = roll(1, 6)
    one_value, = roll(1, 6)
    number = "#{ten_value}#{one_value}"

    isBefore = (ten_value < 4 )
    infos = isBefore ? infos1 : infos2

    typeText = (isBefore ? typeText1 : typeText2)

    index = (one_value - 1) + (isBefore ? (ten_value - 1) : (ten_value - 4)) * 6
    debug("index", index)

    resultText = ''
    if typeText.nil?
      resultText = getReactionTextFull(infos, index)
    else
      info = infos.find{|i| i[:type] == typeText }
      return nil if info.nil?

      resultText = getReactionTex(info, index)
    end

    return "#{name}(#{number}) ＞ #{resultText}"
  end

  def checkTypeText(typeText, infos)
    return true if typeText.nil?

    keys = infos.collect{|i| i[:type] }
    return keys.include?(typeText)
  end

  def getReactionTextFull(infos, index)
    resultTexts = []

    infos.each do |info|
      resultTexts << getReactionTex(info, index)
    end

    return resultTexts.join('／')
  end

  def getReactionTex(info, index)
    typeName = info[:name]
    text = info[:table][index]

    return "#{typeName}：#{text}"
  end

  def getReactionInfoBlood
    infos =
      [
       { :type => "D",
         :name => "ドラク",
         :table => %w{
空に輝く紅い月を仰ぐ
小さくため息をつく
相手を見下すように見る
じっと目を合わせ語りかける
かすかに微笑んで見せる
懐から蝙蝠が飛び立つ
頭上を蝙蝠が渦巻くように飛ぶ
眉を寄せると共に、目が紅く光る
手の中のワインを一口すする
全身から薄紅色のオーラを発する
不敵な笑みを浮かべる
風もなく髪が舞う
飽きた目で相手を見据える
足元に小さなつむじ風が起こる
激情や緊張感に髪が逆立つ
紅い月の光を浴び、目を閉じる
武器を掲げ、誓いの音頭を取る
己の紋章をじっと見つめる
}},
       { :type => "R",
         :name => "ローゼンブルク",
         :table => %w{
鼻で笑う
前髪をかき上げる
己の髪をいじる
一人小さく笑う
ハミングで歌を口ずさむ
片足を軸に、くるりと回る
花弁を具現化し散らす
衣装や鎧の色や装飾を変える
己に酔い、目を閉じる
具現化した華を手の中で弄ぶ
気になる相手を口説こうとする
肩をすくめる
己の武器に口づけをする
誘うように誰かの手を取る
楽器を具現化し音を奏でる
相手にウィンクする
衣装についた埃を払う
マントから無数の蝶が飛び立つ
}},
       { :type => "H",
         :name => "ヘルズガルド",
         :table => %w{
自粛を呼びかける咳払いをする
眉をひそめる
小言を言う
無表情で相手を観察する
足元に小さな地獄門が生じる
周囲にわずかな怨嗟の声が響く
咎めるような目を向ける
直立不動の姿勢で立つ
呆れた様子でため息をつく
突然、振り返って睨む
周囲の空気が凍りつく
胸に手を当て、己を落ち着かせる
手の中に具現化した鎖を弄ぶ
周囲を厳しい目で見渡す
仲間に疑いの視線を向ける
感情に合わせ周囲に鎖が具現化
堕落について忠告する
冷たい視線で相手を一瞥する
}},
       { :type => "M",
         :name => "ダストハイム",
         :table => %w{
眉を寄せて考え込む
周りを値踏みする目で見る
手元に本を具現化し書き込む
つまらなそうに眺める
落ち着いた歩調で近づく
目を閉じ、過去に想いを馳せる
歩む跡、かすかな霧が舞う
苦笑を浮かべて歩み寄る
現状を分析する独り言を言う
興味深そうに質問する
目の前の様式に文句を言う
無感動に会釈をする
手の中に具現化したペンを弄ぶ
目を見開き、感心する
本、袖、マントなどで口元を隠す
宙に浮かび、すべるように進む
無感情に状況を検分する
一瞬、姿が霧に包まれぼやける
}},
       { :type => "A",
         :name => "アヴァローム",
         :table => %w{
欠伸を噛み殺す
頭をかく
手元に生じた果実を食べる
人懐っこい笑みを浮かべる
夜鳥が肩にとまって来る
黒猫が足元にじゃれて来る
周囲を妖精の光が舞う
瞬時に相手の側に現れる
瞬時に相手から離れる
虫や植物に気を取られる
無垢な笑みを見せる
うっかり誰かを巻き込んでころぶ
誰かに甘えるようにもたれかかる
笑みを浮かべながら頷く
小さく首をかしげる
無数の黒い鳥の羽根が宙を舞う
果実を取り出して食べる
思わせぶりな言葉や仕草をする
}},
       { :type => "N",
         :name => "ノスフェラス",
         :table => %w{
冥王領の方角を睨む
舌打ちをする
俯いて床や地面を睨む
無意識に涙が伝う
唇を噛む
胸を押さえじっと考える
首を打ち振って邪念を払う
内心の怒りに険しい貌となる
毅然とした態度で立ち向かう
渇いた笑い声を漏らす
己の血族を貶める言葉を紡ぐ
静かな怒りと共に目が青白く光る
野の花に触れ、それを枯らす
疲れた様子で淡く光る吐息を吐く
己の紋章をぼんやりと見つめる
自嘲的に小さく笑う
疲れた目でぼんやりと他を見る
何か決心したように顔を上げる
}},
       { :type => "G",
         :name => "ゲイズヴァルト",
         :table => %w{
相手を見下すように見る
激情に合わせ頭髪が
高らかに自らの血統を名乗る
武器を具現化し、掲げる
他者を睨みつける
相対する者が、圧迫感を感じる
じっと目を閉じる
他に威圧的な態度をとる
不敵な笑みを浮かべる
片目に手を当て、眉を寄せる
目が一瞬だけ異様に光る
鋭い視線を周囲に巡らせる
髪が淡く光る
背後の者が、貴卿の視線を感じる
眼球の白目が異様な色に光る
目隠しを具現化して目を隠す
余裕のある笑みを浮かべる
目から光が炎のように噴き上がる
}},

       { :type => "K",
         :name => "カインシルト",
         :table => %w{
両手で顔を覆う
逆立つ小さく肩を落とす
過去を思い出し、眉をひそめる
疲れ切ったように座り込む
己の紋章に触れ、ためいきをつく
ワインの水面をじっと見つめる
疲れ、渇いた笑みを浮かべる
居心地悪そうに下を向く
故郷の方角をちらりと見る
衣装の襟や裾をいじる
他の騎士から少し距離をとり、離れる
他の騎士を、自虐的に褒める
胸に手を当て、沈思黙考する
己のかつての血統の名を呟く
沈痛な面持ちで空を見上げる
誇りを持って、己の紋章を掲げる
己の罪や信仰を吐露する
血統を捨てた理由について語る
}},

      ]

    return infos
  end

  def getReactionInfoStyle
    infos =
      [
       { :type => "F",
         :name => "領主",
         :table => %w{
相手の目を覗き心を探る
己の領地に想いを馳せる
従者を侍らせる
従者の世話を受ける
好ましい相手を手招きする
憂いに満ちた物思いにふける
重々しく頷く
優しげな微笑を浮かべる
ちらりと己の紋章を示す
ワインの杯を他の騎士に渡す
じっと風景を眺める
手の中にチェスの駒を具現化する
胸を張り、自信を持って発言する
速やかに謝罪する
他の騎士を正面から褒める
他の騎士の髪や頬を撫でる
重々しく名乗りを上げる
口元を隠しつつ上品に笑う
}},
       { :type => "G",
         :name => "近衛",
         :table => %w{
主の傍にそっと寄り添う
紋章の刻まれた盾を掲げる
武器を構えて上品に一礼する
己の盾の紋章を指でなぞる
他の騎士に寄り添う
主の背後から相手を睨む
主の前、または傍らにひざまずく
きまり悪そうに顔を赤く染める
誰かの前に立ちはだかる
落ち着かなげに歩き回る
はにかんだ笑みを浮かべる
緊張した視線を周囲に巡らす
心の中で他の騎士と試合をする
じっと動かず控えている
主をじっと見つめる
主を褒めたたえる
主の袖を握りしめる
主に対する、周囲の態度を咎める
}},
       { :type => "R",
         :name => "遍歴",
         :table => %w{
かつての戦いについて語る
風に髪をなびかせる
具現化した乗騎を撫でる
仲間の肩を叩く、あるいは抱く
爽やかに笑う
高らかに名乗りをあげる
周囲に積極的に話しかける
己の名に誓い、約束をする
大げさに誰かを褒めたたえる
他の騎士に手合わせを乞う
マントを大げさにひるがえす
相手を褒めそやし口づけを求める
己の故郷を思い出す
己の紋章について語る
他の騎士と世間話をする
困ったように小さく唸る
格上の前に跪いて礼を尽くす
格下や人間に微笑みかける
}},
       { :type => "W",
         :name => "賢者",
         :table => %w{
相手を推し測るように見つめる
メガネを具現化してかける
わずかな点に目を留める
大げさにため息をついて見せる
目を閉じて思索にふける
他の騎士に助言する
ごまかすように咳払いする
わずかな会釈をする
他の騎士に目で合図をする
謎めいた笑みを浮かべる
空に浮かぶ星々を眺める
深い知識で詳しい説明をする
未来について占ってみる
ささやかな予言をする
噂の類を他の騎士に囁く
ふとした品を興味深く観察する
月を見上げドラクルを褒め讃える
他の騎士の感情をいさめる
}},
       { :type => "J",
         :name => "狩人",
         :table => %w{
“敵”を思い出し険しい目になる
唾を吐く
具現化した武器を撫でる
陰鬱な目で虚空を睨む
裾から現れた蛇か蜘蛛を撫でる
思いつめた目で夜空を見る
己の武器を舐めて見せる
暗い笑みを浮かべる
物陰に隠れる
他の騎士の紋章を観察する
空気を読まない発言をする
突然振り向いて背後に警戒する
過去の無念に血涙を流す
鮫のように笑う
その場にいない騎士を嘲笑う
他の騎士とのへだたりを感じる
ぼそりと名乗りをあげる
格下を見下した目で見る
}},
       { :type => "N",
         :name => "夜獣",
         :table => %w{
獣のような荒い呼吸をする
潤んだ上目遣いで相手を見る
俯いて深呼吸する
壁や床に爪を立てて引っかく
己に言い聞かせる独り言を言う
堕落の兆しをじっと見る
他の騎士から目をそらす
他の騎士の機嫌を窺う
相手を睨み唸る
己の指先をちろりと舐め上げる
他の騎士から距離を取り離れる
獰猛な笑みを浮かべる
己の肌を、爪などで傷つける
こっそり舌なめずりをする
妖しい流し目を送る
哀しげな目で己の紋章を見る
獲物を狙う目で、他の騎士を見る
自虐的な言葉をつぶやく
}},
       { :type => "E",
         :name => "将軍",
         :table => %w{
机や壁を叩きつつ、激昂する
失敗した部下をたしなめる
凶悪な笑みを浮かべる
音を立てて踵をそろえ、直立する
不機嫌そうに酒杯をあおる
壁や卓上に周辺地図を具現化する
口をつぐみ、厳めしい顔をする
部下から情報を耳打ちされる
他の騎士を品定めするように見る
内密の話のため、移動する
他の騎士に助言する
他者をいさめる
目を閉じ、戦場に想いを馳せる
部下や民に酒食をふるまう
かつての戦場について語る
戦術について他の騎士と語る
他者を激励する
部下や民からの報告を聞く
}},
       { :type => "B",
         :name => "僧正",
         :table => %w{
深々とため息をつく
紅い月を眺め、涙を溢れさせる
他者を咎め説教する
その場に跪き、月へ祈る
他の騎士を心配する
教典の一文を読み上げる
他の騎士や民を諌める
教典を具現化し、他者に手渡す
真祖へ感謝の言葉を述べる
壇上などに真祖の印を具現化する
騎士や民を微笑み眺める
他者の言葉に大いに感動する
他の騎士の行いを戒める
相手を慈しむ微笑みを浮かべる
他者を大いに褒める
他者をいたわるように撫でる
真祖の印を強く握りしめる
天の紅き月へ感謝を捧げる
}},
       { :type => "H",
         :name => "空駆",
         :table => %w{
乗騎が他を退け、過剰に守ろうとする
粗相した乗騎を、軽く叱る
他の者らを警戒する乗騎をなだめる
他の者に心許す乗騎に嫉妬する
己の乗騎と、子供のように戯れる
乗騎に乗り、共に駆ける素晴らしさを語る
高らかに、己と乗騎の名乗りをあげる
つい礼法を忘れ、誰かの髪を撫でる
乗騎にもたれかかり、目を細める
周りと話しつつも、己の乗騎を気に掛ける
己の乗騎に語りかける
己の乗騎と、食事を分け合う
己の乗騎を褒められて照れる
己の乗騎の頼もしさについて語る
様々な装飾を具現化し、己の乗騎を飾る
具現化ではない小鳥や小動物が集まる
乗騎を抱きしめて撫でながら褒める
己の乗騎を得た次第について語る
}},
       { :type => "K",
         :name => "船長",
         :table => %w{
少しばかり下世話な冗句を言う
礼式に肩をすくめ、居心地悪そうにする
かすかな潮風を具現化させ、吹かせる
口笛で素朴な曲を披露して見せる
瀟洒ならずとも快活な笑いをこぼす
その周囲にかすかな潮騒が聞こえ始める
異国風の歌や曲を披露して見せる
誰かに、美しい貝殻を土産として手渡す
誰かの手を取り、異国風の踊りに誘う
異郷の珍味、酒を周りに勧める
己の船の小さな模型を具現化して見せる
情熱的に、誰かをかき口説く
他の領地で見聞した話を語る
己の船と、その名の由来について語る
誰かを次の航海に誘う
海の脅威について法螺交じりに語る
異郷の様々な情景を具現化して見せる
遥か遠い奇妙な島や海域について語る
}},

       { :type => "L",
         :name => "寵童",
         :table => %w{
主の影にそっと隠れる
主から離れず、ぴったりと密着する
主の袖を小さく摘まみ、軽く引っ張る
主に頭を撫でてもらう
主の髪に指を絡め、梳いたり弄ったりする
主に甘えるようにもたれかかる
主に、ちょっとした我儘を言う
はにかんだ笑顔を浮かべて見せる
目を潤ませ、熱い溜息をつく
きまり悪そうに頬を赤く染め、顔を背ける
申し訳なさそうな顔で、うつむいてしまう
主以外の騎士を、じっと見つめてしまう
無意識に誘うような仕草をしてしまう
誰かに飛びつき、しがみついて抱きしめる
庇護欲をそそらせる微笑を浮かべる
主以外の騎士に、熱心に質問を重ねる
切なげな溜息をつく
主の手を取り、その指を舐める
}},
       { :type => "V",
         :name => "仲立",
         :table => %w{
他者を冷たい目で品定めする
人目を避けて、露骨に不機嫌な顔になる
音を立てずに舌打ちする
ほがらかな笑顔だが、目は笑っていない
敢えて周囲に軽んじられるよう、振る舞う
顎に手を当て、軽く考え込む
会話に入り込めない者を話に誘う
他の者らを紹介させ合い、縁を取り持つ
目論見を思案し、悪い顔で目を細める
周囲の人間模様をそれとなく観察する
誰かに積極的に話しかける
笑顔で会話しつつ、相手の真意を探る
そっと相手の唇に指を当て、言葉を封じる
様々な人々との談笑にふける
誰かにちょっとした贈り物を手渡す
誰かに相談事はないかと、軽く聞く
誰かについて、惜しみない賞賛を語る
誰かを嫌味でない程度に褒めそやす
}},
       { :type => "U",
         :name => "技師",
         :table => %w{
持ち歩いていた人形の部位を弄ぶ
過去の作品を具現化し、自画自賛する
己の技巧の未熟さについて嘆く
己の作品以外が褒められるを聞いて嫉妬
ふとした品を興味深そうに分解し始める
人形に活かすべく誰かの体を撫で回す
誰かの手を取り、弄り回す
突然、羊皮紙に図面を書き始める
メガネを具現化してかける
人形の美しさについて、周囲の意見を問う
相手の美しさを奇妙な視点で賛辞する
思いついた人形の部位を具現化してみる
他愛ない事象から、新たな着想を得る
自動人形のための衣装を具現化してみる
魂と堕落について、誰かと語り合う
己の理想とする人形について語る
誰かの美しさに目を奪われる
誰かの仕草に、嘆息して感動を覚える
}},
       { :type => "D",
         :name => "博士",
         :table => %w{
素晴らしい着想を得て、奇声をあげる
誰かの非理論的な発言に食ってかかる
他の騎士の非合理的作法を鼻で笑う
突然、満面の笑みを浮かべる
己の研究について嬉々として語る
平然と場の空気を無視した発言をする
実験動物を見る目で、誰かを見つめる
衣装の機能性について、誰かを批判する
己の才に酔い、一人ほくそえむ
閃いた着想を紙片に書き留めておく
行き詰った研究について独り言を呟く
誰かのあまりに低い知的水準を嘆く
メガネを具現化してかける
黙り込んで様々な試行錯誤に専念する
目を閉じて、学術的瞑想にふける
ふとした品を興味深く観察し、分析する
誰かに研究への協力を頼みこむ
己なりに知的な言葉で誰かを賛辞する
}},
       { :type => "S",
         :name => "星読",
         :table => %w{
空に同調しすぎて、一瞬だけ具現化が解ける
目を開き空を見上げたまま、仰向けに横たわる
じっと空を見上げたまま、反応しなくなる
星辰の変化に驚愕し、奇声をあげてしまう
ほんやりと空の星を見上げる
星を掴もうとするように両手を広げ、空を見上げる
目を見開き、じっと空の星を集中して見つめる
星辰について感覚的な言葉を並べ続ける
ぶつぶつと星の在り方について呟き、計算する
手近な誰かに、星辰の在り様を細かく説明する
思わぬ星の動きを見つけ、興奮する
誰かに、興味深い星が宿っていることに気づく
誰かに、星の定めた運命を予言する
誰かに、その運命を変えると宣言する
空を指差し、その先にある星を輝かせる
空に手を軽く振り、風を起こして雲をどかす
空に手を軽く振り、雲で特定の星を隠す
空に手を差し出し、紅月をさらに輝かせる
}},
       { :type => "G2",
         :name => "後見",
         :table => %w{
己の過去に搦めて、誰かを説教してしまう
庇護対象の未熟さに、小さな溜息をつく
少し離れた場所から庇護対象を見守る
昂り、出しゃばってしまいそうな己を抑える
庇護対象の背後に、黙って控える
誰かと語らいつつ、庇護対象を時折うかがう
若き騎士らの様子を、まぶしそうに見守る
己の紋章を撫でて、若き日々に思いを馳せる
そっと、庇護対象を背後から抱き寄せる
顎に手を当て、軽く考え込む
会話に入り込めない者を話に誘う
他の者らを紹介させ合い、縁を取り持つ
誰かに、庇護対象の良き逸話を話す
ふと、自身の騎士道の果てが近づいたと感じる
民らの様子を、注意深く観察する
庇護対象の成長に、、思わず顔をほころばせる
庇護対象に、控えめな助言を与える
庇護対象に微笑みかけ、わずかに褒める
}},
      ]

    return infos

  end

  def getHeresyReactionInfoBlood
    infos =
      [
       { :type => "L",
         :name => "異端卿",
         :table => %w{
床を睨み意識を何とか保つ
歩いた跡に、消えゆく塵を遺す
鎧全体に、光が脈打つ
金の軋むような呻き声を漏らす
足音に悲嘆がこもる
声なき咆哮を放つ
他者を威圧する
胸に手を当て、己を落ち着かせる
一瞬、かつての紋章が浮かぶ
鎧の中で金属の軋む音がする
鎧に一瞬、無数の棘が生える
兜の中の目がギラギラと光る
体が痙攣したように震える
鎧の一部が一瞬、武器化する
鎧が蠢いて奇怪な形になる
周囲に青白い光が立ち昇る
己の目的について語る
恐ろしい目で他者を見る
}},
       { :type => "V",
         :name => "ヴルコラク",
         :table => %w{
己の頭をかきむしる
膝をついて、苦しみもがく
一瞬、人狼の姿が重なる
他者の喉を凝視する
獣のように呼吸が荒くなる
己に言い聞かせる独り言を言う
唇を噛み、口の端から血を流す
ざわざわと衣服の下で肉が蠢く
牙を剥きだして唸る
顔を険しくしかめる
思いつめた目で押し黙る
壁にすがりつき、嗚咽を漏らす
己の胸を押さえて深呼吸する
獲物を狙う目で、騎士を見る
月をじっと見つめる
一瞬、頭に狼の耳が現れる
月を見上げ、怒りの唸りを漏らす
決意した顔で、正面を見る
}},
       { :type => "N",
         :name => "ナハツェーラ",
         :table => %w{
生きたものをじっと見つめる
己の手を見て、深いため息をつく
少しだけ遅れて受け応えする
一瞬、脱力して死体に戻る
ひからびたような笑い声を漏らす
歩いた跡に血の足跡が残る
周囲に生ぬるい湿った風が吹く
手がどろりと血に塗れる
無意識に、涙が流れてしまう
植物に触れ、一瞬で枯らす
裾から多数の骨が溢れ出す
触れた通常の品が、朽ち果てる
周囲の温度が下がる
目が青白く光る
淡く光るため息を漏らす
傍らに鬼火を具現化する
生前について語る
手の中に髑髏を具現化する
}},
       { :type => "K",
         :name => "カルンシュタイン",
         :table => %w{
姿が霧に包まれぼやける
一瞬、白目が黒く染まる
存在感を消して物陰に佇む
周囲の些細な音に警戒する
騎士から目をそらす
一人小さく笑う
床を浮かび、すべるように進む
歩む跡、かすかに黒い霧が舞う
黒い蒸気のような吐息を吐く
片足を軸に、くるりと回る
己の髪をいじったり、かき上げる
衣装についた埃を払う
口元を隠して笑みをこぼす
他から距離を取って、闇に潜む
騎士の紋章をじろじろと見る
貴族的な笑みをこぼす
自信に溢れた様子で胸を張る
豪奢な衣装を具現化してまとう
}},
       { :type => "G",
         :name => "グリマルキン",
         :table => %w{
その場で居眠りする
大きな欠伸をする
猫の姿で、居眠りをする
猫の姿で、壁をひっかく
他者の具現化に飛びかかる
黒猫が足元にじゃれて来る
尻尾をぴんと伸ばす
猫の姿で、他者を威嚇する
不満を漏らす
猫の姿で、狭い所に入り込む
耳と尻尾がぴくぴくと動く
家具や木の上に登る
猫の声で鳴く
退屈そうに小物をいじり回す
周囲に近隣の猫が集まる
猫の姿で、他者の膝上にのる
適当な小物にじゃれついて遊ぶ
他者にじゃれつく
}},
       { :type => "S",
         :name => "ストリガ",
         :table => %w{
翼をきつく閉じて怯える
他の騎士や異端にとにかく謝る
弱者をいじめる
他のストリガを貶める言葉を紡ぐ
強者にへりくだって媚びを売る
民などに高慢な態度をとる
民の小さな失敗をなじる
肩にとまった小鳥と語らう
翼を広げ、他者に手をさしのべる
袖から小鳥が飛び立つ
ハミングで歌を口ずさむ
上空で鳥が輪を描き、歌う
他者に愛想よく微笑みかける
無数の黒い鳥の羽根が宙を舞う
他者を翼で包み抱きしめる
美しい声で歌い始める
肩に舞い降りた鳥が耳元で鳴く
周囲に無数の小鳥が集まる
}},
       { :type => "M",
         :name => "メリュジーヌ",
         :table => %w{
ぼんやりと遠くを眺める
口から小さく威嚇音を漏らす
一瞬舌を出し、香りを舐めとる
つまらなさそうに様子を眺める
過去を思い出すように考え込む
目を見開き、他者に感心する
湿った風が渦巻く
下半身を人間のものに変える
手から水を生み出し、杯に注ぐ
周囲に水たまりができる
謎めいた笑みを浮かべる
舌を伸ばし、他者の顔を舐める
下半身の蛇を、他者に絡ませる
興味深そうに質問する
毒蛇を具現化し、身に絡め弄ぶ
袖から無数の蛇が現れる
卵を取り出して丸呑みする
他者に下半身を絡め抱きしめる
}},
       { :type => "F",
         :name => "フォーン",
         :table => %w{
より退廃的な衣装をまとう
悪戯っぽく首をかしげてみせる
よこしまな笑顔を見せる
ふしだらな舌なめずりをする
顔を上気させ、ため息をつく
己を抱きしめて身をよじる
己の指先をちろりと舐め上げる
ひどく淫らな声を漏らす
武器にいやらしく舌を這わす
濡れた上目遣いを見せる
扇情的な仕草をする
頭髪があやしく波打つ
妖しい流し目を送る
他者の耳元に囁き、息をかける
他者にしなだれかかる
他者の体に手を這わせ愛撫する
謎めいた笑みを浮かべる
他者を口説き始める
}},
       { :type => "H",
         :name => "ホムンクルス",
         :table => %w{
苦しそうに咳き込みながら膝をつく
ふらつき、壁や他の騎士にもたれる
己の胸に手を当て、沈痛な顔を見せる
腕か脚が一瞬、霧のように崩れかける
自嘲的な乾いた笑いが漏れる
無表情に周りの様子をぼんやり眺める
月光を避けるように影や屋内に入る
他の騎士に手を伸ばしかけて、止める
不意に月光を浴びてしまい、苦痛を耐える
会話に入ろうとするが、やめてしまう
具現化した花の消える様子に己を重ねる
他の騎士たちをまぶしそうに見守る
未来について、詳細な推理を述べる
疲れた様子で、淡く光る吐息を深々と吐く
瞬時に予想し、落ちかけた品を受け止める
遠い先の話題に、自虐的な笑みを漏らす
一刻後の天候を、正確に言い当てる
ほんのかすかな笑顔が浮かぶ
}},
       { :type => "E",
         :name => "エナメルム",
         :table => %w{
人形の虚ろな表情からは何もわからない
居心地悪そうに、隅にいる
何かを口にしかけるが、うつむいてしまう
何かに触れ、カチンと金属音を立てる
ぎこちない動きで体を動かす
誰かに無言で、何かを訴える
身振り手振りで、己の感情を伝えようとする
せめて感触を得ようと、誰かに触れ続ける
誰かをじっと、硝子の瞳で見つめている
人の体にはできない動きで、奇妙に舞う
立ったまま、微動だにしない
己の記憶を取り戻そうと、辺りを見回す
無意識に、表情も動かぬまま涙が溢れる
誰かの所作を、己に取り入れようとする
声ではない、楽器の音色のような歌を披露
無機的な目で、冷たく誰かを一瞥する
無言でただそっと、誰かに寄り添う
断片的に、かつての魂の記憶が蘇る
}},
       { :type => "S2",
         :name => "サングィナリエ",
         :table => %w{
怒りや欲望で、顔が怪物のように醜くゆがむ
衝動的に誰かを抱き寄せ、首筋に顔を寄せる
騎士を嘲るように見て、鼻で笑う
渇きと共に、口の端から紅い唾液が垂れる
足元から真紅の血だまりが広がり、場を覆う
目が光り、誰かの首筋を舐めるように見る
騎士道の偽善性を説き、嘲笑う
誰かの首筋を、じいっと凝視する
胸に手を当て、深呼吸して己を落ち着かせる
誰かの血の味を口の中に思い出し、ほくそ笑む
切なげなため息をつき、舌なめずりする
誰かに、慇懃無礼な皮肉めいた誉め言葉を送る
マントをひるがえして誰かに近づくか、傍から退く
目が異様な光を灯す、民がいるなら陶然となる
わざとらしく衣装についた埃を払ってみせる
高らかに、サングィナリエの血統を名乗る
騎士の在り方について、異端の論を披露する
余裕のある不敵な笑みを浮かべる
}},
       { :type => "A",
         :name => "アールヴ",
         :table => %w{
平然と場の空気を無視した発言をする
他の誰かが褒められて様子に、嫉妬する
ハミングで、妖精の歌を口ずさむ
好奇心も露に、誰かに話しかける
肩にとまった小鳥と語らう
野の花をそっと愛でて、微笑む
誰かに、剣での立ち合いを求める
流れるような動きで誰かに近づき、手を取る
妖精の美酒を手元に出し、誰かの盃に注ぐ
周囲に小さな妖精たちが、光の点となって舞う
妖精郷の歌や舞を披露してみせる
様々な人々との談笑にふける
誰かに愛の言葉を捧げ、口説き始める
誰かについて、惜しみない称賛を語る
少し奇妙な言い回しで、誰かを賛辞する
騎士道について、熱く語る
己の属する妖精郷の宮廷に、誰かを誘う
誰かの振る舞いに嘆息し、大いに感動を覚える
}},
       { :type => "V2",
         :name => "ヴィーヴル",
         :table => %w{
周囲の者は、圧迫感と居心地の悪さを感じる
誰かに、理不尽な我儘を言う
つまらなさそうに他の者達の様子を眺める
話題を断ち切り、己の目的を率直に遂げんとす
傲慢とも取れる目で誰かを見下ろし、品定めする
威嚇するように、腕に翼を大きく広げる
誰かを子供扱いし、わざと挑発してからかう
幻に包まれたように、その姿が一瞬だけ揺らぐ
表情を変えずとも、ぞわりと龍体の色が変わる
浮かんだまま、遠い目をする
他の者達の絆の強さを、見定めようと観察する
誰かに髪の手入れを求める
各地の様々な事象について、次々と質問する
真祖と龍と月について、滔々と語り始める
誰かの行いを諫め、戒める
誰かを龍の翼で包むように抱きしめ、褒める
誰かを慈しむように撫で、いたわる
その姿が一瞬、巨大な龍に見える
}},
       { :type => "L2",
         :name => "ルーガルー",
         :table => %w{
巨大な狼の姿となって、警戒も露にうなる
怒りを耐えるように、拳をきつく握りしめる
苛立たしげに、狼の口から牙を剥きだしてうなる
怒りを抑えるが、獣毛が逆立つ
疲労や緊張から、獣のように呼吸が荒くなる
疲れ、乾いた笑みを浮かべる
周囲の些細な音に警戒し、耳を動かす
誰かを睨みつけ、威圧する
狼の鼻先が、ひくひくと動く
鋭い視線を無表情で巡らせ、状況を把握する
目を閉じ、腕を組んで、じっと立ち続ける
1人遠く、故郷の方角へと視線を向けて佇む
決意した顔で、正面から誰かと向かい合う
沈痛な面持ちで空を見上げ、うなる
自然の在り様について、言葉少なに語る
たとえ屋内であろうと、野の風が吹く
誰かの美しさに、思わず目を奪われる
目を閉じて、故郷たる自然に思いを馳せる
}},
       { :type => "A2",
         :name => "アルラウネ",
         :table => %w{
不満を漏らしたり、軽い我儘を言ったりする
そっと誰かの耳元に何かを囁く
己の髪に指を絡め、梳いたり弄ったりする
感じ取った未来を口にしかけるが、やめてしまう
1悪戯っほい、謎めいた笑みを浮かべる
誰かに己の身をすりつけるように、絡みつく
瞬時に予知し、落ちかけた品を受け止める
わずかに動くごとに、周囲を花の香りが包む
舞うような足取りで、誰かに密着するよう近づく
煽情的な仕草をしつつ、妖しい流し目を送る
誰かに甘えるようにもたれかかる
誰かの運命について、断片的に口にする
足元に妖しくも美しい花畑が具現化される
具現化した蔓と葉が、その肢体に絡みつく
周囲に無数の花弁を具現化しつつ、舞う
より華麗で美しい衣装をまとう
未来について、曖昧な予言を口にする
己に咲く花を摘み、誰かに手渡す（瞬時に再生）
}},
       { :type => "F2",
         :name => "フリッガ",
         :table => %w{
その影が巨大な蜘蛛となって、蠢く
他の騎士や異端にとにかく謝る
居心地悪そうに、隅にいる
ほがらかな笑顔だが、目は笑っていない
恋の話題に、自虐的な笑みしか浮かばない
己に口い聞かせるように、独り言を言う
会話に入ろうとするが、やめてしまう
申し訳なさそうな顔で、うつむいてしまう
無意識に、己の腹をいとおしげに撫でる
誰かの腕や袖をしっかりと掴んで、離さない
誰かを思いつめた目で、じっと見つめている
目を潤ませ、熱い溜息をつく
他の騎士に手を伸ばしかけて、止める
具現化した糸を、誰かの指とつなぐ
困ったような、泣き笑いの微笑みを浮かべる
誰かにびったりと密着して離れない
そっと相手の唇に指を当て、言葉を封じる
誰かに、狂気の如き恋心を抱き始める
}},
      ]

    return infos
  end

  def getHeresyReactionInfoStyle
    infos =
      [
       { :type => "W",
         :name => "野伏",
         :table => %w{
露骨に、騎士から距離をとる
騎士や民を見て、舌打ちする
騎士について不満を口にする
民の在り様について批判する
獰猛な笑みで周りをねめつける
粗野に体をぼりぼりと掻く
床に唾を吐き捨てる
武骨な武具を具現化する
革袋の酒や水を飲み干す
他者に咎めるような目を向ける
手づかみで、粗野に食事する
騎士を見下した目で見る
正面から相手を見据え会話する
取り出した肉を噛みちぎり喰らう
自然の力について語る
周囲を厳しい目で見渡す
無礼な発言をする
己の土地について語る
}},
       { :type => "N",
         :name => "流浪",
         :table => %w{
呆れた様子でため息をつく
欠伸を噛み殺す
大げさに肩をすくめて見せる
他から距離を取り、様子を眺める
具現化した果実を食べる
口笛を吹く
一組のカードを具現化して弄ぶ
名もなき野草の花を愛でる
楽器を具現化し、音を奏でる
他者と世間話をする
風評を元に他者を褒めたたえる
吹く風に目を細める
小鳥や虫、植物らに話しかける
仲間の肩を叩く、あるいは抱く
風に髪をなびかせる
旅の土産話を披露する
目を閉じてリラックスする
他者に異郷の珍しい品を渡す
}},
       { :type => "S",
         :name => "密使",
         :table => %w{
周りを値踏みする目で見る
冷たい目で外を眺める
濁った眼で、他者をじっと見る
無感情に状況を検分する
小さな名もなき花を目に留める
他の騎士に目で合図を送る
感情のない目で他者を見る
小さく自嘲的な笑みを浮かべる
過剰なほどに周囲を警戒する
他者をちらちらと観察する
手の中でナイフを弄ぶ
騎士に深々と礼をする
自虐的な言葉を呟く
そっと騎士に寄り添い控える
他者と世間話をし、談笑する
主の名を、心の中で唱える
騎士を褒めたたえる
他者に積極的に話しかける
}},
       { :type => "H",
         :name => "魔女",
         :table => %w{
誰もいないところに話しかける
含み笑いから、やがて高笑いへ
眉を寄せて、水面を見つめる
騎士を胡乱な目で見る
大鍋を具現化し、かき混ぜる
胡散臭い笑みを浮かべる
足元をゴブリンが駆けていく
小さく含み笑いを漏らす
奇妙な言い回しの言葉を言う
具現化した杖で、床を叩く
思わせぶりな言葉や仕草をする
周囲を魔法や妖精の光が舞う
奇妙な知識で状況説明する
奇怪な呪文を呟く
あやしげな護符を取り出す
袖から現れた小動物と会話する
他者の人相などを占う
手の中に青白い炎を具現化する
}},
       { :type => "F",
         :name => "剣士",
         :table => %w{
騎士の礼儀作法を嘲笑い、茶化す
具現化した武器の柄に手を伸ばす
凶悪な笑みを浮かべ、相手をねめつける
油断なく周囲に視線を走らせ、警戒する
剣を手の中に具現化し、弄ぶ
口笛やハミングで、簡単な曲を奏でる
不敵な笑みを浮かべながら、名乗る
周囲の騎士や異端との戦いを、思い描く
誰かに関する風聞と、実際の印象を語る
警戒と同時に、手が武器に伸びてしまう
自信に溢れた様子で、堂々と会話する
過去の敗北から得た教訓について語る
闘志により周りの風景が、ぐにゃりと歪む
周囲に風が渦巻き、衣装をはためかせる
気に入った相手を口説こうとする
勇ましく迫力に満ちた剣舞を披露する
騎士や民に目を向けて、舌打ちする
かつての戦いや冒険について語る
}},
       { :type => "X",
         :name => "検体",
         :table => %w{
肉体が異形化しようとするように、波打つ
発作的にうずくまり、呻き苦しむ
指の長さが異様に伸びたり縮んだりする
ふとした金属音に、異様なほど怯える
汗をかきながら、俯いて深呼吸する
己に言い聞かせるように、独り言をいう
蘇る苦痛の記憶に、震えが止まらない
誰かにぴったりとしがみついて離れない
一瞬、肌に無数の傷跡が浮かび上がる
過去を振り払うように、首を打ち振る
初対面の人物の所作に、怯え隠れる
体の内から、脈打つように異様な光を放つ
一瞬、爪や犬歯が伸びて異形の姿となる
何かの拍子に、異様なほどの跳躍をする
他の者を妬み呪う言葉が口から溢れる
目から血が涙のように流れ出す
壁にすがりつき、何かを抑え嗚咽を漏らす
誰かに、己の苦しみからの助けを求める
}},
      ]

    return infos

  end

  def getCorruptionDiceCommandResult(command)

    return nil unless /^CT(\d+)$/ === command

    modify = $1.to_i

    name = "堕落表"
    table =
    [
     [ 0, "あなたは完全に堕落した。この時点であなたは［壁の華］となり、人狼、黒山羊、夜獣卿のいずれかとなる。その［幕］の終了後にセッションから退場する。247ページの「消滅・完全なる堕落」も参照すること。"],
     [ 1, "あなたの肉体は精神にふさわしい変異を起こす……。「堕落の兆し表」を2回振って特徴を得る。このセッション終了後、【道】を「夜獣」にすること。（既に「夜獣」なら【道】は変わらない）"],
     [ 3, "あなたの肉体は精神にふさわしい変異を起こす……。「堕落の兆し表」を1回振って特徴を得る。このセッション終了後、【道】を「夜獣」にすること。（既に「夜獣」なら【道】は変わらない）"],
     [ 5, "気高き心もやがて堕ちる。あなたが今、最も多くルージュを持つ対象へのルージュを全て失い、同じだけのノワールを得る。ノワールを得た結果【渇き】が3点以上になった場合は、再度堕落表を振ること。"],
     [ 6, "内なる獣の息遣い……あなたが今【渇き】を得たノワールの対象へ、任意のノワール2点を獲得する。"],
     [ 7, "内なる獣の息遣い……あなたが今【渇き】を得たノワールの対象へ、任意のノワール1点を獲得する。"],
     [ 8, "荒ぶる心を鎮める……幸いにも何も起きなかった。"],
     [99, "あなたは荒れ狂う感情を抑え、己を律した！　【渇き】が1減少する！"],
    ]

    number, number_text = roll(2, 6)
    index = (number - modify)
    debug('index', index)
    text = get_table_by_number(index, table)

    return "2D6[#{number_text}]-#{modify} ＞  #{name}(#{index}) ＞ #{text}"
  end

  def getTableDiceCommandResult(command)

    info = @@tables[command]
    return nil if info.nil?

    name = info[:name]
    type = info[:type]
    table = info[:table]

    text, number =
      case type
      when '2D6'
        get_table_by_2d6(table)
      when '1D6'
        get_table_by_1d6(table)
      when 'D66'
        get_table_by_d66(table)
      else
        nil
      end

    return nil if( text.nil? )

    return "#{name}(#{number}) ＞ #{text}"
  end

  def getSourceSceneDiceCommandResult(command)
    return nil unless /^ST$/ === command

    name = "原風景表"
    table = getSourceSceneTable
    return getTitleD66TableResult(name, table)
  end

  def getPassionDiceCommandResult(command)
    return nil unless /^PN$/ === command

    name = "受難表"
    table = getPassionTable
    return getTitleD66TableResult(name, table)
  end

  def getTitleD66TableResult(name, table)
    debug("getTitleD66TableResult")
    item, index = get_table_by_d66(table)
    return nil if item.nil?

    title, text, = item

    return "#{name}(#{index}) ＞ #{title}：#{text}"
  end

  def getConfermentDiceCommandResult(command)
    return nil unless /^CO$/ === command

    name = "叙勲表"
    table = getConfermentTable
    yearTitle = "叙勲年齢"

    return getYearTableResult(name, table, yearTitle)
  end

  def getConfermentAfterDiceCommandResult(command)
    return nil unless /^CA$/ === command

    name = "叙勲後表"
    table = getConfermentAfterTable
    yearTitle = "騎士歴"

    return getYearTableResult(name, table, yearTitle)
  end

  def getElderPastDiceCommandResult(command)
    return nil unless /^EP$/ === command

    name = "遥か過去表"
    table = getElderPastTable
    yearTitle = "騎士歴"

    return getYearTableResult(name, table, yearTitle)
  end

  def getOriginalSinDiceCommandResult(command)
    return nil unless /^OS$/ === command

    name = "原罪表"
    table = getOriginalSinTable
    yearTitle = "外見年齢"

    return getYearTableResult(name, table, yearTitle)
  end

  def getRecentStateDiceCommandResult(command)
    return nil unless /^RS$/ === command

    name = "近況表"
    table = getRecentStateTable
    yearTitle = "活動年数"

    return getYearTableResult(name, table, yearTitle)
  end

  def getPeacefulPastDiceCommandResult(command)
    return nil unless /^PP$/ === command

    name = "平和な過去表"
    table = getPeacefulPastTable
    yearTitle = "外見年齢"

    return getYearTableResult(name, table, yearTitle)
  end

  def getYearTableResult(name, table, yearTitle)
    item, index = get_table_by_d66(table)
    return nil if item.nil?

    title, text, yearText, = item
    year, calculateText = getYear(yearText)

    return "#{name}(#{index}) ＞ #{title}：#{text} ＞ #{yearTitle}：#{yearText} ＞ #{calculateText} ＞ #{yearTitle}：#{year}年"
  end

  def getYear(yearText)
    text = yearText.gsub(/(\d+)D(6+)/){ getD6xResult($1.to_i, $2.length) }
    text = "(#{text})"

    year = parren_killer( text.gsub(/×/, "*") )
    return year, text
  end

  def getD6xResult(count, dice6Count)
    total = 0

    count.times do |i|
      number = 0

      dice6Count.times do |i|
      number *= 10
        dice, = roll(1, 6)
        number += dice
      end

      total += number
    end

    return total.to_s
  end

  def getSourceSceneTable
    table =
      [
       ["虐待", "あの地獄のような日々で、貴卿がされてきたことを思えば……騎士となっても、人を見る目は暗い。"],
       ["野良犬の如く", "孤児だった貴卿はひたすら盗み殺し騙して。手段を問わず生き抜いた。騎士の前には罪人として突き出された。"],
       ["悪行", "貴卿は叙勲以前、悪しき不徳の行いに手を染めていた。幸いにも主には気づかれず、騎士となったが……行いは正されているだろうか？"],
       ["忌まわしの火", "故郷は太陽の欠片に襲われた。家族は命を落とし、騎士が貴卿を救った。あの呪わしい炎を貴卿は忘れない。"],
       ["太陽の夢", "今はなき太陽の照らす……美しく生命に満ちた世界を、幼い頃から夢で見る。奇妙な疑念が内から消えない。"],
       ["黒い衝動", "暴力性や残虐性が、貴卿の中に眠っている。それは時折突発的に現れて……大事なものを破壊するのだ。"],
       ["氷の心", "過酷な日々を暮らす中、貴卿の心は凍てついた。能力を買われ騎士となるが……心の氷はまだ溶けない。"],
       ["腰抜け", "かつての貴卿は、立ち向かうべきものから逃げ出した。それは騎士となってなお残る、心の疵だ。"],
       ["孤独", "貴卿はずっと孤独だった。手を差し伸べてくれたのは主のみ。主なき日々など考えられない。"],
       ["野心", "貴卿はより高い地位が欲しい。民では足りない。騎士になりたい。もっともっと多くを支配したいのだ。"],
       ["惚れっぽい", "簡単に恋をしてしまう。わずかな縁でも恋人のつもりになって、心から尽くしてしまうのだ。"],
       ["間近の堕落", "故郷の領主が堕落し、暴虐を為した。貴卿は、騎士というものについて他ほど無邪気には信じられまい。"],
       ["妄想", "夢見がちな貴卿は、自らが騎士となる夢想を何度もしてきた。今も、ともすれば夢を見ているのだろうか。"],
       ["空白", "かつての時間の中には思い出せない時期がある。その間に何があったのか、貴卿には何もわからない。"],
       ["敬虔なる心", "貴卿は人の頃より紅き月とドラクルを深く信仰してきた。たゆまぬ信仰こそ、己を騎士に導いたと信じている。"],
       ["妖精の導き", "入ってはならぬ森で妖精と親しくなった。そして戯れる中、気づけば貴卿は騎士の城にいた。"],
       ["従者の一族", "生まれつき従者として生きる一族もいる。貴卿はその中に生まれ、才見出されて騎士となったのだ。"],
       ["市井の心", "ごく凡俗な民の家にて長く育った貴卿は世俗的ながらも、民の視野を持つ騎士となるであろう。"],
       ["流浪", "貴卿は人の頃から流浪の身。視野を広げるつもりの旅であったが、従者としてこの地に居つくこととなった。"],
       ["人の血縁", "血縁者が騎士となった。そしたまた貴卿も……。貴卿には、人としての血縁者が、同胞にいるのだ。"],
       ["妖精の友", "妖精を引き寄せる、奇妙な魅力を持って生まれた。幼き頃より、貴卿は妖精や獣の友である。"],
       ["無垢なる魂", "浮世の邪念から余りに遠く生きて来たがゆえに、貴卿の魂はあまりに気高く美しい。そして脆い。"],
       ["戦場の獣", "主と共に、貴卿は数多の戦場を渡った。戦いは貴卿の呼吸であり歩み。何処にあろうとも戦いの嵐は逃さぬ。"],
       ["古き格式", "貴卿は非常に格式を重んじる地に生まれ育った。たとえ形骸化しようと、古き格式を忘れて何が騎士か。"],
       ["カリスマ", "人に頼られ注目を浴びやすい。貴卿は生まれつき、人々のまとめ役だ。騎士には必要な才と言えるだろう。"],
       ["鍛錬の日々", "騎士となる日を目指し、幼き頃から武の鍛錬欠かさず。いまや貴卿はこれほどの武勇を得たのだ。"],
       ["恩返し", "かつて、貴卿はある騎士に助けられた。そして彼の方を助けるべく従者となり……ついに騎士となったのだ。"],
       ["騎士の養子", "幼くして見いだされ、物ごころついた頃には、騎士手づからの教育と鍛錬を受けて来た。人の親は知らない。"],
       ["英雄志願", "守られる民としての生活に耐えられなかった。貴卿は英雄への道を求め家を飛び出し、従者となったのだ。"],
       ["燃える恋", "一目見て、ある騎士に恋をした。人の身では叶わぬ恋。しかし貴卿はついに叙勲を受け、騎士となる。"],
       ["救世の想い", "人々は様々な恐怖に怯え、困窮している。己の努力で何とかできれば……と、貴卿は騎士への道を選んだ。"],
       ["美貌", "貴卿の美貌が評判になる頃、その美を耳にした騎士が訪れ、貴卿を城に導いた。"],
       ["憧れの騎士", "貴卿には憧れる騎士がいる。いつかきっとその人と同じ高みに立つのだと、心に決めた人物がいるのだ。"],
       ["ドラクルの夢", "ある日の夢の中、貴卿のもとにドラクルが降りた。そして人生の全てを変える神託を賜ったのだ。"],
       ["神童", "時として、貴卿のように有り余る才を持って生まれる者がいる。そして定められたように騎士への道を歩むのだ。"],
       ["運命の子", "貴卿の生まれた折、旅の賢者が訪れた。貴卿はきっと伝説に残る騎士となるだろう。そう約束されているのだ。"],
      ]
    return table
  end

  def getConfermentTable
    table =
      [
       ["堕落の血", "貴卿は堕落者に襲われ、その結果……騎士となった。貴卿に流れるは堕落した血なのだ。", "7+1D6×1D6"],
       ["忌まわしの血", "乱心した主は堕落の間際、公爵の許しも得ず貴卿を叙勲した。残された貴卿には……疑いの目が向けられた。", "7+1D6×1D6"],
       ["悪しき取引", "貴卿は主と悪しき取引の後、叙勲を受けた。主は不徳の身、今や貴卿の立場は主の奴隷も同然である。", "7+1D6×1D6"],
       ["裏切の予言", "叙勲の折、賢者は貴卿に予言をした……いずれ信じた者に裏切られる日が来るであろうと。", "7+1D6×1D6"],
       ["謀略", "貴卿は騎士にあるまじき行いによって他を蹴落とし、叙勲を得た。その秘密を知る者はいない。今のところは。", "7+1D6×1D6"],
       ["不吉な予言", "叙勲の折、賢者は貴卿に予言をした……大切な人を自らの手で殺めるだろうと。", "7+1D6×1D6"],
       ["堕落の誘い", "その叙勲の折、貴卿の脳裏に堕落した己が現れた。そして、堕落すべしと囁きかけて来た。今も、時折。", "7+1D6×1D6"],
       ["不相応", "他の従者はみんな優れた才能を持っていた。貴卿には何もない……なのに主は、なぜか貴卿を選んだ。", "10+2D6"],
       ["矮小なる主", "叙勲と共にものの見方は変わる。叙勲後、主がくだらない存在と見えた。貴卿は主のもとを離れた。", "7+1D6×1D6"],
       ["家庭あれど", "貴卿は人として家庭を築いていた。しかし、主より叙勲を受けられるならばと……貴卿は家庭を捨てたのだ。", "14+1D6×1D6"],
       ["かき口説き", "貴卿は主を甘き言葉で口説き、心蕩かして叙勲を与えさせた。精神面において主はむしろ……貴卿なのだ。", "7+1D6×1D6"],
       ["不安", "騎士とは何か。叙勲とは何か。騎士そのものとなった今も、貴卿は言い知れぬ不安を感じている……。", "10+1D6×1D6"],
       ["忘却", "誰が貴卿を騎士にしたのか。正直わからない……気が付いたら騎士になっていたのだ。", "7+1D6×1D6"],
       ["ある夜の寝室", "それは衝動的な行いだったのだろう。主はある夜、突如訪れ……あなたに口づけをし、叙勲を施したのだ。", "10+2D6"],
       ["生存者", "堕落者、あるいは太陽の欠片の襲撃を受け、主の従者は全滅。貴卿のみが生き延び……叙勲を受けた。", "7+1D6×1D6"],
       ["献身の誓い", "叙勲の折、貴卿は民を守り続けると誓った。貴卿は民の盾にして、数多の敵を退ける矛である。", "14+1D6×1D6"],
       ["罪許されて", "貴卿は罪を犯した。罪人として処罰を受ける中、主は貴卿を認め叙勲すら与えてくれた。報わねばなるまい。", "10+2D6"],
       ["守秘の誓い", "叙勲の折、貴卿は主から秘密を知らされた。そしてそれを決して他に漏らさぬ誓いを立てたのだ。", "7+1D6×1D6"],
       ["勝者は一人", "その日、主の従者数多において武術の試合が行われた。貴卿は最後に勝ち残り、ついに叙勲を得たのだ。", "14+1D6×1D6"],
       ["磨かれし技", "今まで一つの技を磨き、極め続けた。その愚直に磨かれた技が評価され、貴卿は叙勲を受けたのだ。", "18+1D6×1D6"],
       ["武勲の予言", "叙勲の折、賢者は貴卿に予言をした……いずれ大乱が起こる中にて目覚ましき武勲を上げんと。", "10+2D6"],
       ["癒えぬ傷痕", "貴卿の体には消えぬ傷痕が刻まれている。しかし、傷を受けてまで主を守った功こそ、貴卿を騎士にした。", "7+1D6×1D6"],
       ["隆盛の予言", "叙勲の折、賢者は貴卿に予言をした……その名は財と誉れに包まれ安らかにあらんと。", "7+1D6×1D6"],
       ["胆力", "貴卿は主と共に堕落者に襲われ……一歩も退かずあれば、その胆力を認められ叙勲を受けた。", "7+1D6×1D6"],
       ["友情の予言", "叙勲の折、賢者は貴卿に予言をした……いずれ素晴らしき友を得んと。", "7+1D6×1D6"],
       ["惜しまれり", "長く従者として生きる中、貴卿は老いた。しかし、老いてなお、主は貴卿を必要とした。それゆえの叙勲である。", "30+1D6×1D6"],
       ["守護の誓い", "叙勲の折、ある人を、町を、国を守ると誓った。それを背に敵に立ち向かう限り、あなたは逃げ出したりしない。", "14+1D6×1D6"],
       ["返しきれぬ恩", "あの日、貴卿は死んだ……。しかし貴卿は、主の叙勲により蘇った。主は罰を受けたが、それでも笑っていた。", "7+1D6×1D6"],
       ["信仰の誓い", "叙勲の折、貴卿はドラクルへの篤き信仰を誓った。紅き月の下、貴卿の勝利はドラクルに捧げられよう。", "14+1D6×1D6"],
       ["終わらじの恋", "騎士と互いに恋に落ちた。甘く狂おしい時間、移ろいゆく時。貴卿は恋人の手で叙勲を受け、騎士となった。", "10+2D6"],
       ["英雄の予言", "叙勲の折、賢者は貴卿に予言をした……恐るべき敵を打ち倒し、その名は諸国に響くであろうと。", "14+1D6×1D6"],
       ["忠義の誓い", "叙勲の折、ある人に深き忠誠を誓った。たとえ世界を敵に回そうと、貴卿が裏切ることなどありはしない。", "14+1D6×1D6"],
       ["生ける芸術", "叙勲により、貴卿の美貌はさらなる高みに上った。もはや貴卿を見て心奪われぬ者などいない。", "7+4D6"],
       ["思わぬ武勲", "貴卿は従者の身ながら、思わぬ武勲を立てた。その功は叙勲を受けるに十分のものである。", "14+1D6×1D6"],
       ["当主の寵愛", "貴卿は、公爵家当主その人の寵愛を得た。宮廷において、重き地位を得られるであろう。", "7+1D6×1D6"],
       ["始祖の寵愛", "貴卿は、始祖その人の寵愛を得た。真祖ドラクルに最も近しい、誇るべき身の一人となったのだ。", "7+1D6×1D6"],
      ]
    return table
  end

  def getConfermentAfterTable
    table =
      [
       ["滅亡", "貴卿の領地は敵に滅ぼされた。守るべき民は絶え、名誉は失われた。彼の者らを許すわけにはいかない。", "2D6×10"],
       ["冤罪", "貴卿は身に覚えのない罪を背負っている。何者かの陰謀が貴卿を絡めとり、罪を負わせたのだ。", "1D6×1D6"],
       ["呪い", "かつて貴卿は、妖精から呪いを受けた。呪いはゆっくりと貴卿を蝕み、その身に災いを招いている。", "1D6×1D6"],
       ["血の昂ぶり", "戦った後は体が火照り昂ぶり、どうしようもなく疼く。堕落の日は遠くない……貴卿の中には確信がある。", "2D6×5"],
       ["報復の序章", "かつて貴卿の名誉は友に裏切られ、奪われた。今や新たな道を歩んでいるが、復讐は今も忘れていない。", "2D6×10"],
       ["仇敵", "忘れてはならぬ敵がいる。やつの胸を貫き、首を落とすまで、貴卿に安息の日は来ない。来ないのだ。", "1D6×1D6"],
       ["堕落の共鳴", "貴卿の魂は堕ちかけているのだろうか。堕落した者たちがどうしようもなく近しく思えてしまうのだ。", "2D6×10"],
       ["不徳の日々", "かつての貴卿は品行方正とは言い難い人物だった。心入れかえた貴卿だが、まだまだ世間の目は厳しい。", "1D6×5"],
       ["主の堕落", "貴卿を叙勲した主が堕落した。主を誇って来た貴卿は、心ない他の騎士から、侮蔑や疑惑を受けがちだ。", "2D6×10"],
       ["冥王の虜囚", "冥王の侵攻の中。貴卿は敗れ、一時なれど虜囚の辱めを受けた。奴らを許すわけにはいかない。", "2D6×3"],
       ["近しき堕落", "貴卿の友や恋人が堕落した。それは決して拭えず、忘れられない。常に心に幽鬼の如く頭をもたげるのだ。", "1D6×1D6"],
       ["修羅", "貴卿は戦い続けて来た。功を他に譲ってでも戦い、戦い、戦い続けて。戦うだけの存在となりつつある。", "1D6×1D6"],
       ["空白", "騎士となってから、思い出せない時期がある。その間に何があったのか、貴卿には何もわからない。", "2D6×10"],
       ["追跡", "堕落した友を捜している。彼の者が大切だからこそ、貴卿がケリをつけねばならないのだ。", "2D6×10"],
       ["退屈", "変わらぬ日常。甲斐なき日々。仲間の武勲を聞き、貴卿は自らも勲を立てんと立ち上がった。", "2D6×20"],
       ["失恋", "愛深きゆえに貴卿は愛を捨てた。……捨てきってはいないかもしれないが、捨てたと信じた。", "2D6×10"],
       ["隠遁", "戦いに虚しさを感じたのはいつからか。勇猛な騎士だったあなただが、今は静かに隠者として暮らしている。", "2D6×20"],
       ["謎の失踪", "貴卿の大切な人が突然、姿を消してしまった。攫われたのか、逃げたのか、それともまさか……。", "1D6×1D6"],
       ["井の中の蛙", "己こそ最高の騎士と思っていた貴卿だが……彼の騎士を見て全ては崩れた。まだまだ修練が足りぬ。", "1D6×3"],
       ["賢者の予言", "貴卿は予言に導かれている。貴卿は誰かと出会わねばならぬ。それは主君か友か恋人……あるいは敵か。", "1D6×1D6"],
       ["浮名", "多くの恋を振りまく貴卿の名は宮廷では有名だ。敵も味方も、きっと抱えきれぬほどにいることだろう。", "1D6×5"],
       ["見届け人", "民の移り変わる様をじっと眺めている。貴卿は目まぐるしく変わる彼らを眺めるのが好きなのだ。", "2D6×10"],
       ["人捜し", "貴卿はその心を動かした人を捜している。人間だ。人は脆く移ろいやすい……早く見つけねば。", "1D6×1D6"],
       ["深い寵愛", "貴卿は主より深い愛情を注がれ続けた。そのため叙勲は随分以前だが、騎士の活動は始めたばかりだ。", "2D6×10"],
       ["血気盛ん", "貴卿は多くの騎士と立ち合い、自らの技と力を磨きたい。腕のある騎士を見れば、挑まずにいられぬ。", "2D6"],
       ["啓蒙", "至高なる騎士と違い、世界は愚かで哀れな民で溢れている。騎士として彼らを導いてやらねばなるまい……。", "1D6×1D6"],
       ["叙勲すべし", "必要な人がいる。彼の人を叙勲せねばならぬ。貴卿はそのため、功に焦り始めている。", "2D6"],
       ["憂国の士", "貴卿は己が国の現状を憂う。宮廷の問題を正さねばならぬと、心に深く誓っている。", "1D6×1D6"],
       ["叙勲与えり", "功少なくともそれなりの古株として、貴卿は己の従者を叙勲し、騎士に取り立てた経験がある。", "2D6×20"],
       ["堕落の虜囚", "貴卿は一時、堕落者の虜囚となり、慰み者として日々を過ごしたことがある。屈辱であり汚名だ……。", "2D6×10"],
       ["名勝負", "貴卿は己の意志を貫くため決闘し、勝利した。その決闘は、近隣でも噂の名勝負とされている。", "1D6×1D6"],
       ["長き眠り", "貴卿はかつて太陽の欠片と戦い、体の多くを喪失して眠りについた……随分と長く寝ていたようだ。", "2D6×50"],
       ["世界は広く", "騎士となって世界の広さを知った。可能な限り世界を知りたい……貴卿は広き世界を見聞せんとしている。", "2D6×10"],
       ["王器", "叙勲以来、人をひれ伏させる圧倒的な魅力を得た。それはまさしく、支配者となるべく与えられた資質だ。", "1D6×1D6"],
       ["高名", "数多の決闘や馬上試合で実力を示してきた。あなたは実力ある騎士として名を馳せている。", "2D6×5"],
       ["伝説", "貴卿は未だ偉業わずかなれど、その逸話は伝説の域。すでに吟遊詩人らの歌にさえ歌われている。", "1D6×1D6"],
      ]

    return table
  end

  def getElderPastTable
    table =
      [
       ["古き罪人", "地獄が生まれるより前。誰もが忘れた牢に閉じ込められていた罪人……それが貴卿だ。幾百年を経て、ついに牢は破られた。", "1D66+1300"],
       ["地獄漏れ", "貴卿は不徳と堕落を重ね、地獄へと幽閉された……はずだ。しかし、気づけば外におり、数百年が過ぎていた……。", "1D666"],
       ["禁忌の探求", "とうに死した一人の民……あるいは騎士を、貴卿は蘇らさんとしている。外法の類だ。わかっていても……諦められぬのだ。", "1D666"],
       ["堕落せし身", "一度は堕落し、恥と穢れにまみれた。心ある友らが救い出してくれたが、汚名は今も付きまとう。堕落の兆しが全て消えようとも。", "1D666"],
       ["臆病の失態", "太陽竜の現れた時、貴卿は恐怖から討伐の招集より逃げ出した。この恥は、長き時を経た今も貴卿の心を苛んでいる。", "1D66+1250"],
       ["無限の復讐", "貴卿はこの世の理不尽に泣かされた。ゆえに終わらぬ復讐を始めたのだ。この世界全てを、変えてしまうために……。", "1D666"],
       ["悪しき発端", "かつて貴卿は、よからぬ考えに取りつかれていた。かつての活動は、今も悪しき後継者を生んでいる。止めねばなるまい。", "3D6×100"],
       ["妖精の輪", "妖精に誘われ、一夜の宴を楽しんだ。そして今……かつてより、幾百年が過ぎている。貴卿を知る騎士は、まずいない。", "2D6×100"],
       ["破滅の疫病", "あの頃、疫病はノスフェラス以外も侵した。貴卿の故郷もまた……民を見殺しにした貴卿に、彼の冥王を責められようか。", "1D66+1210"],
       ["主の堕落", "貴卿を叙勲した主が堕落した。主を誇って来た貴卿は、己の中から何か大切なものが……すっぽりと抜けたように思えた。", "1D666"],
       ["異端の友", "かつて、異端と騎士の距離は近かった。貴卿には異端の友がおり、距離を取りつつもお互いに助け合って来たのだが……。", "2D6×100"],
       ["時を超えて", "何があったのかわからない。しかし、気づけば遥か未来の大地に立っていた。幾百年もの空白が、今も貴卿を苛んでいる。", "3D6×100"],
       ["太陽竜襲撃", "彼の太陽竜による襲撃を、貴卿はまともに受けた。守るべき領地は失われ、討つべき太陽竜も、己の手では討てなかったのだ。", "1D66+1300"],
       ["数多の堕落", "知る者の多くが堕落していった。かつての友で今も残るはわずか。決して堕落すまいと、固い決意をしているが……。", "2D6×100"],
       ["最後の命令", "太陽竜討伐の折、ある将軍が貴卿を庇い、帰らぬ人となった。最後の命令は「生きろ」――以来、貴卿は命令を守り続けている。", "1D6+1250"],
       ["今は後日談", "貴卿は復讐を成し遂げた。そして、為すべきことを失った。かろうじて堕落は避けているが、新たな生きる甲斐を見つけねば。", "1D666"],
       ["守るべき地", "ある領主が、堕落した。惨劇の後、もはやそこには誰も住まない。だが、貴卿が彼の人と過ごした思い出はある。守らねば。", "1D666"],
       ["開拓者", "民の心を知りたくて。貴卿は一人荒野を拓き耕した。やがて村ができ、貴卿は領主となった。彼の地は貴卿の第二の故郷だ。", "1D666"],
       ["己だけが", "太陽竜討伐の折、目の前でゲイズヴァルト家が滅んだ。貴卿にわずかでも勇気があれば、彼の家は残っていたのでは……。", "1D66+1250"],
       ["信仰の日々", "僧正として拝殿にこもり、大いなるドラクルへの感謝を捧げてきた。今も、紅き月を見上げる時、貴卿の内には感謝が満ちる。", "2D6×100"],
       ["寵童の日々", "叙勲の主によって、屋敷の中で大事に大事に扱われてきた。騎士として不本意でもあったが、人として彼の方を愛している。", "1D666"],
       ["傍観の日々", "民の移り変わる様をじっと眺めている。貴卿は目まぐるしく彼らを、隠者として眺めているのが好きだった。", "3D6×100"],
       ["遍歴の日々", "遍歴の騎士として諸領を巡り、数多の騎士、数多の冒険、数多の恋の中で過ごしてきた。今も色あせぬ美しき思い出だ。", "2D6×100"],
       ["忠義の日々", "仕えるべき御方を見つけ、貴卿は全てを捧げて彼の人を守ってきた。その忠誠は未だ損なわれてはいない……。", "2D6×100"],
       ["民の血統", "叙勲され、騎士になったが……民の頃を捨てられなかった。貴卿はずっと、ずっと、己の血縁や友の子孫を見守っている。", "1D6×150"],
       ["長い眠り", "生き疲れてか、傷を負ってか。貴卿は長い長い眠りについた。そうして近年ようやく目を覚ましたばかりである。", "2D6×100"],
       ["冥王と後悔", "かつてノスフェラスにて地位を得ていた。しかしあの災禍の中で貴卿は逃げたのだ。冥王と化す彼女を見捨てて……。", "1D66+1250"],
       ["勃興の協力者", "貴卿はローゼンブルク勃興のため、グランツ公へと協力してきた。彼の裏も表もよく知っているし、彼の地で勢力も築いている。", "1D66+400"],
       ["ノスフェラス", "貴卿はノスフェラス末期、疫病の中で叙勲を受けて救われたる騎士。冥王とその一党の風評を何より苦々しく見てきた者だ。", "1212"],
       ["財産", "長い年月の中、貴卿は多くの品や建物を具現化し、永続化した。貴卿の周囲は己の生み出したもので溢れている。", "2D6×100"],
       ["妖精の国", "騎士を引退し、歴代アヴァローム公の如く、妖精らの国で楽しく暮らしていた。今宵、騎士として世に出たは、気まぐれに過ぎぬ。", "2D6×100"],
       ["数多の叙勲", "数多の戦いを経て来た貴卿は、数え切れぬ武勲を立てて来た。一つ一つを数えるのも、もはや馬鹿らしいほどだ。", "1D66×10"],
       ["血統の重鎮", "血統とは、騎士の最も大きな枠組みであり派閥だ。そして貴卿は、血統内において大きな発言力を持つ重鎮なのだ。", "3D6×100"],
       ["始祖の懇意", "始祖とて心寂しい時はある。貴卿はある始祖と懇意にしていた。彼女にとって、あなたは数少ない心許せる存在なのだ。", "3D6×100"],
       ["太陽竜討伐", "彼の恐るべき太陽竜討伐に参加し、華々しき活躍を飾った。その一方で貴卿は、多くの犠牲者らも目にしている。", "1D66+1300"],
       ["最初の騎士", "貴卿こそは、真祖百年の喪が明けし時、始祖その人より最初期に叙勲されたる騎士。常夜全ての歴史の生き証人だ。", "1D66+1833"],
      ]

    return table
  end

  def getOriginalSinTable
    table =
      [
       ["虐待", "あの地獄のような日々で、御身がされてきたことを思えば……誰も彼も、常夜国すべて等しく呪うしかない。", "7+1D6×1D6"],
       ["野良犬の如く", "孤児だった御身は盗み殺し騙して。手段を問わず生き抜いた。罪人として捕まらず、逃げ延びたは天運か。", "7+1D6×1D6"],
       ["弱肉強食", "暴力で欲しいものを奪ってきた御身は、略奪者にして狩人。どれだけ取り繕っても、世を支配するのは獣の法。", "7+1D6×1D6"],
       ["黒い衝動", "暴力性や残虐性が、御身の中に眠っている。それは時折突発的に表れて……大事なものを破壊するのだ。", "7+1D6×1D6"],
       ["氷の心", "過酷な日々を暮らす中、御身の心は凍てついた。異端として力を得た今も……心の氷はまだ溶けない。", "7+1D6×1D6"],
       ["生存者", "堕落者、あるいは太陽の欠片の襲撃を受け、故郷は全滅。御身のみが生き延びた。背負う魂は数多い。", "7+1D6×1D6"],
       ["血の渇き", "民の体を流れる血に、異様な執着を覚えてしまう。どうしても満たされぬ時は、己の血を舐めるほどに。", "7+1D6×1D6"],
       ["嘲笑の的", "いつも、周りで誰かが嘲笑う。誰も彼も御身を嘲笑う。何が間違っているというのか、何がおかしいというのか。", "7+1D6×1D6"],
       ["死者を前", "に大切な人が死んだ。朽ちゆく死体を見て、死のおぞましさを知った。何があろうと絶対に、死んでなるものか。", "7+1D6×1D6"],
       ["騎士への嫉妬", "御身は騎士という存在に嫉妬していた。美しく強く老いぬ者ら……なぜ奴らは華々しく、己は惨めなのか。", "13+1D6×1D6"],
       ["背負う罪", "かつて、ふとした悪戯心が、人を死なせてしまった。どれほど時を経ても、最初に刻まれた罪は消えない。", "7+1D6×1D6"],
       ["騎士の堕落", "よく知る騎士が堕落し、暴虐を為した。御身は、騎士というものが異端と何も変わらぬと知っている。", "13+1D6×1D6"],
       ["太陽の幻視", "昔から時折、太陽に照らされた大地の夢を見る。それは決して、悪い世界ではなかったと思えるのだが……。", "7+1D6×1D6"],
       ["退廃", "酒色に御身は、どうしようもない屑と化していた。思い出したくもない、怠惰と強欲に溺れた日々。", "13+1D6×1D6"],
       ["虚栄", "誰かに認めて欲しくて、己を嘘で塗り固めて。努力以上に、嘘を重ねて生きて来た。", "13+1D6×1D6"],
       ["黒い歴史", "かつての己は弱く、愚かで。思い返せば、頭を掻きむしりながら消え去りたいという想いばかりが。", "13+1D6×1D6"],
       ["刻まれた恐怖", "恐ろしい存在に出会った。堕落者か、妖精か、太陽か。御身の中には、それに対する恐怖が刻まれたままだ。", "7+1D6×1D6"],
       ["血気盛ん", "性急に過ぎると、古くから言われていた。しかし年を経ても、気質は治らぬ。御身の心は今も、若く、未熟だ。", "7+1D6×1D6"],
       ["孤独への恐怖", "一人きりは怖い。寂しさは恐怖だ。媚びてでも、使われてでも、傍にいてくれないと、気が狂いそうになる。", "10+1D6×1D6"],
       ["野育ち", "森や荒野で一人生きて来た。獣も人も何も変わりはしない。ただ一匹の獣として生きて来たのだ。", "7+1D6×1D6"],
       ["唸る獣", "人に、威圧的に接してしまう。友好的に接したくても、それができない。御身の笑顔は牙を剥く獣の如し。", "15+1D6×1D6"],
       ["移り気", "どうにも目先のものに気をとられがちだ。落ち着いてものごとを考えられない。年経ても変わらぬ性分だ。", "6+2D6"],
       ["空白", "遠い昔、異端の力を得る前の己について、覚えていない。思い出そうとすると、脳中に霧がかかるのだ。", "7+1D6×1D6"],
       ["蘇生の願い", "大切な人が死んだ。けれど諦められない。諦めない。何か、解決策が、蘇らせる方法が、きっと。", "7+1D6×1D6"],
       ["本の虫", "静かに本を読むのが好きだ。本の中の世界、己の世界に没頭したい。外で駆けまわるのは苦手なのだ。", "13+1D6×1D6"],
       ["老醜", "老いて朽ちゆく恐怖。死への忌避感。必死でもがいて足掻いて、御身はそれから逃れようとした。", "35+1D6×1D6"],
       ["好奇心", "気を引くものがあれば、目を向ける。秘密とあれば暴かずにいられない。知らぬままでは、済ませられぬのだ。", "9+2D6"],
       ["惚れっぽい", "簡単に恋をしてしまう。わずかな縁でも恋人のつもりになって、心まで尽くしてしまうのだ。", "13+1D6×1D6"],
       ["叶わぬ初恋", "ある民に恋をした。恋は叶わず、彼の人は御身の前から姿を消した。けれど、あの笑顔は、今も御身を力づける。", "9+2D6"],
       ["無垢なる魂", "浮世の邪念から余りに遠く生きて来たがゆえに、御身の魂はどこまでも穢れなく美しい。そして脆い。", "6+2D6"],
       ["平穏", "御身は家族に囲まれ、愛情に満ちた生活を送っていた。平凡な生活、平凡な日々。とても、とても幸せだった。", "7+1D6×1D6"],
       ["甘い夢", "世界を何も知らなかった甘い夢の日々。その夢から覚めたのは幸福か、不幸か。あるいは今も、夢の中か。", "7+2D6"],
       ["美貌", "誰もが羨み、振り返る美貌。その美貌は周囲に響き、また騎士すらも御身を賛美した。", "7+1D6×1D5"],
       ["妖精の友", "妖精を引き寄せる、奇妙な魅力を持って生まれた。幼き頃より、御身は妖精や獣の友である。", "7+1D6×1D6"],
       ["騎士の奉仕者", "御身は麗しき騎士に仕えていた。信頼と憧れを育み、その立場に誇りを持っていたのだが……。", "13+1D6×1D6"],
       ["神童", "御身は有り余る才を持って生まれた。騎士になるかと思われていたのだが……。", "7+1D6×1D6"],
      ]
    return table
  end

  def getPassionTable
    table =
      [
       ["地獄", "騎士らによって、御身は一度は地獄に封じられた。何とか脱出できたのは、幸運以外のなにものでもあるまい。"],
       ["暴発する殺意", "殺すつもりなどなかったのに。ふとした口論から、かっとなって……一人の民をその手で、殺めてしまった。"],
       ["復讐の道", "かつて御身は友に裏切られ、全てを奪われた。今や御身の命は、かつての友を葬るためにある。"],
       ["異端審問", "あの忌まわしき異端審問官らに捕まり、屈辱と苦痛を受けた。彼の者らの存在など決して認めぬ……。"],
       ["仇敵", "忘れてはならぬ敵がいる。やつの胸を貫き、首を落とすまで、御身に安息の日は来ない。来ないのだ。"],
       ["堕落の冤罪", "一人の騎士が堕落した。その罪を民や他の騎士らは、当人ではなく……異端である御身に押し付けたのだ。"],
       ["忌まわしの火", "故郷は太陽の欠片に襲われた。全てが焼かれ、御身も傷を負った。あの呪わしい炎、決して忘れはしない。"],
       ["惨劇に佇む", "気がつけば、無数の民の血肉舞う惨状の中に立っていた。これは己の所業なのか……？　答える者はいない。"],
       ["異相の拒絶", "かつての御身は己の異端の体を受け入れられなかった。何度も己の異相を切り落とし、堕落の淵に迷った。"],
       ["堕落の道標", "何気ない言葉が、友の心を傷つけた。彼の者は堕落した。御身の言葉が切っ掛けでないと、どうして言えようか。"],
       ["力に酔う", "異端の力を得て、有頂天となった。己こそ王と信じ、民に暴虐を為した。すぐに誤りを思い知りはしたが……。"],
       ["近しき堕落", "友や恋人が堕落した。それは忘れられぬ過去。ともすれば己もまた、と悪しき未来を思い描いてしまう。"],
       ["民への失望", "信頼を育んで来た民が、異端審問官らの言葉に流され、御身を罵り始めた。御身が何をしたというのか。"],
       ["民の罠", "快く迎えられ、宿に泊められ……眠りについた御身を襲ったのは異端審問官だった。もはや、民など信じぬ。"],
       ["異端卿の友", "御身のよく知る異端が、御身の友たる騎士を再叙勲し、異端卿に変えた。御身一人だけが……何も知らされず。"],
       ["堕落の様相", "御身は堕落者に襲われた。何とか生き残ったが、あの怪物の様相は……己もまた、ああなってしまうのか？"],
       ["それでも力が", "御身は騎士になれなかった。それでも力が欲しくて……異端にすがりつき、隷属と引き換えに力を得たのだ。"],
       ["石もて追われ", "人外の力を持つ御身に、民は石を投げつけ追い払わんとした。何もしてはいないのに……民は信じられぬ。"],
       ["呪い", "妖精から呪いを受けた。呪いはゆっくりと御身を蝕み、その身に災いを招く。すでに呪いは解けたろうか？"],
       ["望まぬ救い", "受難より御身を救った者は、さらなる災いであった。救い手は暴君となり、御身の全てを支配したのだ。"],
       ["汚い仕事", "騎士に雇われ、下劣な仕事をした。結果、汚名をかぶせられ、その地を逃げ出す羽目となったのだが。"],
       ["異端の住処", "行ってはならぬ場所に行き。逢ってはならぬ異端に逢って。御身はそのまま、故郷へ帰れぬ身となった。"],
       ["死別", "特別な存在だった民が……少し見ぬ間に死んでいた。同じ異端に変え、時を過ごすつもりだったのに。"],
       ["真の恋", "これまでの恋とはただの欲望であった。御身は真の恋を知ったのだ。全てが手遅れになった後に……。"],
       ["拒絶", "愛した民がいた。同じ異端に変えて、長い時の伴侶としたかったが……当人から拒絶された。"],
       ["最期の足掻き", "民として死にかけ。潔く散るはずが……最後の最後で執着に囚われた。代価を差し出し、異端と化したのだ。"],
       ["死の味", "きっと御身は一度は死んだのだ。命を失い、体が冷たい塊と化すあの時を……とてもよく、覚えているのだから。"],
       ["妖精の導き", "入ってはならぬ森で妖精と親しくなった。そして戯れる中、気づけば見知らぬ土地と時代にいたのだ。"],
       ["つまずき", "ほんのわずかなつまずきが、いつまでも澱のように残っている。この憂いが晴れる日は来るのだろうか。"],
       ["忘却", "御身に何か忌まわしいことが起きた……はずだ。しかし思い出せない。いったい何が……？"],
       ["人でなしの恋", "御身は異端との恋に落ちた。甘く狂おしい時間、移ろいゆく時。御身は恋人の手で、異端の体を得た。"],
       ["主の気まぐれ", "あの日、死んだはずの御身……。しかし、通りすがった主が、御身を異端へと変えて救ってくれたのだ。"],
       ["悲劇の予言", "魔女が御身に予言をした……いつか、大切な人を自らの手で殺めるだろうと。"],
       ["かき口説き", "御身は、孤独な異端を甘き言葉で口説き、心蕩かし、己を仲間に加えさせた。精神面において主はむしろ……。"],
       ["孤独", "ずっと孤独だった。異端となってなお孤独。御身はこのまま、堕落を待つばかりかと思ったのだが……。"],
       ["井の中の蛙", "己こそ最強と思っていた御身だが……一人の騎士を見て、全ては崩れた。まだまだ、世界は広い。"],
      ]
    return table
  end

  def getRecentStateTable
    table =
      [
       ["悪夢", "眠りにつくたび、受難の記憶が蘇る。御身の心は晴れず、忌まわしい悪夢がつきまとい続けているのだ。", "2D6×10"],
       ["血の昂ぶり", "戦った後は体が火照り昂ぶり、どうしようもなく疼く。堕落の日は遠くない……御身には確信がある。", "1D6×1D6"],
       ["逃走", "異端審問官に追われ、住み慣れた地を離れねばならなかった。民も、異端に不審の目を向ける。生きづらい。", "1D6×1D6"],
       ["堕落の共鳴", "御身の魂は堕ちかけているのだろうか。堕落した者たちがどうしようもなく近しく思えてしまうのだ。", "1D6×1D6"],
       ["死にたがり", "もう飽きるほど生きた。堕落者や太陽を少しでも巻き添えに……己にふさわしい場所で、消滅してしまいたい。", "3D6×30"],
       ["再叙勲の儀", "騎士を異端卿に変える方法を、御身は知っている。そして実際に一人の異端卿を生み出してもいるのだ。", "3D6×30"],
       ["改心", "かつての御身はろくでもない悪党だった。心入れかえたのだが……異端に対する世間の目は厳しい。", "2D6×10"],
       ["追跡", "堕落した友を探している。彼の者が大切だからこそ、御身の手でケリをつけねばならないのだ。", "2D6×10"],
       ["嵐の予感", "御身には予感がある――常夜国に近く、恐るべき災厄が起きるであろう。少しでも多くに知らせ、備えねば。", "2D6×10"],
       ["仮面", "傷つくのが怖くて、表情を作った。本当の己を隠した。今や御身はもう、本当の己を忘れてしまっている……。", "2D6×10"],
       ["墓守", "親しくして死んでいった者たちの墓を見守っている。誰もが拝殿の墓場に葬られるわけではないのだ。", "2D6×20"],
       ["空白", "その体となってから、思い出せない時期がある。その間に何があったのか、御身には何もわからない。", "1D6×1D6"],
       ["失恋", "御身は異端ゆえか、その性格ゆえか。一つの恋を終えた。今は傷心のままに彷徨っている。", "1D6×1D6"],
       ["主がために", "騎士か異端か。御身には、心に決めた特定の主がいる。その方の命に従い、御身は行動する。", "1D6×1D6"],
       ["新米", "御身は未だ、異端となったばかり。得た力に振り回され、受難にも懲りず（古き異端は選択できない）。", "1"],
       ["退屈", "変わらぬ日常。甲斐なき日々。ゆっくりと朽ちていく感覚。このまま己を失い、堕落してしまうのだろうか。", "2D6×10"],
       ["隠遁", "民の目、騎士の目は、平穏を乱す。御身は自らの平穏のため、厳しき自然の中で隠棲している。", "2D6×20"],
       ["愛がほしい", "寂しくて人恋しい日々。誰かを信じたい。誰かに信じられたい。利用されてもいいから、人の側にいたい。", "1D6×1D6"],
       ["魔女の予言", "御身は予言に導かれている。御身は誰かと出会わねばならぬ。それは主か友か恋人……あるいは敵か。", "1D6×1D6"],
       ["見届け人", "民の移り変わる様をじっと眺めている。御身は目まぐるしく彼らを眺めるのが好きなのだ。", "3D6×30"],
       ["後継探し", "御身は己の心を動かす民を探している。己と同じ存在に変え、永遠の時を共に過ごすのだ……。", "3D6×20"],
       ["ゴブリン", "ゴブリンたちに懐かれている。彼らは頻繁に、御身の前へと姿を現し、ちょっとした悪戯や贈り物をしていくのだ。", "1D6×1D6"],
       ["血の後継", "御身は目に留めた者を仲間に加えたことがある。そして、その者との親交は未だ途絶えてはいない。", "3D6×30"],
       ["神秘の伝承", "神秘の技を、民や他の異端に伝えている。古い知識を伝えるためあるいは単に驚く顔を見ようと。 ", "3D6×20"],
       ["変革の狼煙", "異端と蔑まれ、このような立場に追いやられた現状を是とするのか？　立ち上がり、変えるべきではないか？", "1D6×1D6"],
       ["民と共に", "長く民と交わり、彼らとの交流の中で生きて来た。多くの感謝と、いくらかの疑いを受けながら生きている。", "1D6×1D6"],
       ["放浪癖", "一つ所に留まる気になれない。御身は常に道を歩き、数多の村、数多の都市を渡り歩いてきた。", "1D6×1D6"],
       ["民の間に", "積極的に民と交わり、彼らの生活に関わっている。御身は人が好きで、彼らと接してこそ、己を感じられるのだ。", "1D6×1D6"],
       ["恩返し", "かつて、御身はある騎士、または異端に助けられた。いつか、この恩は返さねばならない。", "1D6×10"],
       ["騎士の友", "騎士と時を過ごし、信頼を築いた。騎士という連中への忌避感は少ない。彼らとは助け合えるだろう。", "2D6×10"],
       ["長き眠り", "御身はかつて人知れず太陽の欠片と戦い、体の多くを喪失して眠りについた……随分と長く寝ていたようだ。", "3D6×50"],
       ["英雄の星", "出会った魔女が予言した……いずれ御身が恐るべき敵を打ち倒し、その名は諸領に響くであろうと。", "1D6×1D6"],
       ["導くもの", "多くの異端が、蔑まれる立場に甘んじ、堕落していく。御身が誇りと、家族と、愛情を教えねばならない。", "3D6×20"],
       ["慈愛", "御身は全てを慈しむ。たとえ今は不幸な行き違いがあろうとも、彼らを救い続ける手を止めてはなるまい。", "2D6×10"],
       ["生ける芸術", "その美しさゆえ、御身に平穏は訪れぬ。姿を見ただけで、あらゆる者が御身に焦がれ、愛を捧げよう。", "1D6×1D6"],
       ["伝説", "御身は異端なれど、常夜国の歴史に少なからず関わってきた。吟遊詩人にも歌われる伝説の存在だ。", "3D6×50"],
      ]
    return table
  end

  def getPeacefulPastTable
    table =
      [
       ["乱暴者", "短気で、気性が荒く、ついつい手が出て乱暴を働いてしまう。故郷での評判はあまりよくない。今はどうだろう？", "8+2D6"],
       ["孤児", "親兄弟もなく、ただ一人で知恵を働かせて生きて来た。自由だが、厳しい環境の中で御身の魂は磨かれてきたのだ。", "6+2D6"],
       ["絶対の恐怖", "ある生き物が、どうしようもなく恐い。とにかく生理的に無理。それを見れば御身は無様に飛び上がり逃げまどう。", "7+1D6×1D6"],
       ["酒乱", "昔から酒に酔うと、どうにもろくなことをしない。民でなくなった今、酒には酔わないはずだが……。", "15+1D6×1D6"],
       ["怠け者", "才能はあっても、動くのはどうにも億劫で。いつも、どうにかしてラクに過ごそうと、そればかり考えていた。", "7+1D6×1D6"],
       ["内気", "臆病で人見知りする性格。人との交流も苦手。付き合いは深く狭く。そんな御身でも縁はあるもので……。", "7+1D6×1D6"],
       ["血気盛ん", "性急に過ぎると、古くから言われていた。しかし年を経ても、気質は直らぬ。御身の心は今も若く、未熟だ。", "7+1D6×1D6"],
       ["幼馴染", "とても仲のいい幼馴染がおり、将来について語り合っていた。今、彼の人はどうしていることだろうか。", "7+1D6×1D6"],
       ["忘れざる味", "御身には、どうしても忘れがたい大好物がある。民でなくなった今も、それを夢中で口にしてしまうだろう。", "7+1D6×1D6"],
       ["朴念仁", "かつて多数の異性から想いを向けられたが、御身は気づかなかった。最近ふと思い出し、気づいたのだ。", "7+1D6×1D6"],
       ["ある別れ", "親友や家族が、騎士または異端となり……故郷を離れた。しかし、同じ時を生きる今なら、きっと会 えるはずだ。", "7+1D6×1D6"],
       ["本の虫", "静かに本を読むのが好きだ。本の中の世界、己の世界に没頭したい。外で駆けまわるのは苦手なのだ。", "7+1D6×1D6"],
       ["はねっかえり", "どうにも人の言葉を、素直に聞けない。ついつい反抗しては周りに迷惑をかけてしまう。時を経た今も変わらず。", "6+2D6"],
       ["お調子者", "あれこれと失敗しているのに、ついつい調子に乗ってしまう。勢いのままに、無謀な行動をしてしまいがちだ。", "7+1D6×1D6"],
       ["好奇心", "気を引くものがあれば、目を向ける。秘密とあれば暴かずにいられない。知らぬままでは、済ませられぬのだ。", "9+2D6"],
       ["説教癖", "何かと人の所作が気になり、つい人にうるさく言ってしまう。己自身にも厳しくあろうとしているのだが……。", "15+1D6×1D6"],
       ["腕自慢", "恵まれた体躯と身体能力で、故郷において並ぶものなき腕自慢、力自慢だった。無論、民としては……だが。", "9+3D6"],
       ["自然の中で", "故郷が僻地にあったのか。それとも野に暮らしていたのか。御身は自然に溶け込み、自然をよく知っている。", "7+1D6×1D6"],
       ["旅人", "様々な村や街を行き来する旅人だった。流浪の異端や、遍歴の騎士にも、それなりの知り合いがいる。", "10+1D6×1D6"],
       ["自由に焦がれ", "平凡な日々への反発。己という枠を超えて、異端になってしまえば、きっと自由になれると思った。", "9+2D6"],
       ["叶わぬ初恋", "ある民に恋をした。恋は叶わず、彼の人は御身の前から姿を消した。けれど、あの笑顔は、今も御身を力づける。", "9+2D6"],
       ["奇妙な初恋", "一人の異端と出会い、その美しさに恋をした。彼の人を求め、すべてを打ち捨て……同じ時を生きる身となった", "9+3D6"],
       ["移り気", "どうにも目先のものに気をとられがちだ。落ち着いてものごとを考えられない。年経ても変わらぬ性分だ。", "6+2D6"],
       ["鳥獣の友", "鳥や獣の意思がなんとなくわかる。御身は彼らの友であり、様々な場で助けられ、また助けてもきた。", "7+1D6×1D6"],
       ["妖精の友", "妖精を引き寄せる、奇妙な魅力を持って生まれた。幼き頃より、御身は妖精や獣の友である。", "7+1D6×1D6"],
       ["おせっかい", "周りの人が気になって仕方がない。困っている人には、迷惑がられても口出ししてしまう性分なのだ。", "7+1D6×1D6"],
       ["無垢なる魂", "浮世の邪念から余りに遠く生きて来たがゆえに、御身の魂はどこまでも穢れなく美しい。そして脆い。", "6+2D6"],
       ["修道僧", "真祖の拝殿にて、僧正に仕える修道僧だった。もっとも、異端審問のような視野の狭い考えは持っていない。", "10+1D6×1D6"],
       ["平穏", "御身は家族に囲まれ、愛情に満ちた生活を送っていた。平凡な生活、平凡な日々。とても、とても幸せだった。", "7+1D6×1D6"],
       ["働き者", "故郷では評判の働き者。誠実な人柄でも知られていた。その気質は今も変わらず、余裕を持って行動できない。", "12+1D6×1D6"],
       ["熟練者", "ある専門技術について、天才的な能力を持っている。故郷の領主も、御身の意見や技術を頼みとしたほどだ。", "15+1D6×1D6"],
       ["熱愛", "とても深く、激しい、熱病のような恋をした。恋の炎は今も、御身の中でくすぶっている。決して消えてはいない。", "9+3D6"],
       ["美貌", "誰もが羨み、振り返る美貌。その美貌は周囲に響き、また騎士すらも御身を賛美した。", "7+1D6×1D6"],
       ["魔女の弟子", "故郷の近くに住む魔女にかわいがられていた。薬草や占い、異端について、御身は人よりよく知るだろう。", "7+1D6×1D6"],
       ["騎士の友", "ある騎士によく寵愛を受けていた。否、立場の差を超えた、友情や親愛と言ってもいい対等の絆をつないだのだ。", "7+1D6×1D6"],
       ["血統を遺す身", "伴侶を得て、子を為し、家庭を築いた。その血は、御身が民ならざる身となった今も脈絡と続いている。", "12+4D6"],
      ]
    return table
  end

  @@tables =
    {
    'CS' => {
      :name => "堕落の兆し表",
      :type => '2D6',
      :table => %w{
あなたは完全に堕落した。この時点であなたは［壁の華］となり、人狼、黒山羊、夜獣卿のいずれかとなる。その［幕］の終了後にセッションから退場する。247ページの「消滅・完全なる堕落」も参照すること。
獣そのものの頭（狼、山羊、蝙蝠のいずれか）
夜鳥の翼
コウモリの翼
鉤爪ある異形の腕
ねじれた二本角
狼の耳と尾
青ざめた肌
異様な光を宿す目
突き出した犬歯
目に見える変化はない……
},},

    'ECS' => {
      :name => "拡張・堕落の兆し表",
      :type => 'D66',
      :table => %w{
完全なる堕落：あなたは〔壁の華〕となり、いずれかの堕落者となる。その〔幕〕の終了後にセッションから退場する。（基本247ページ参照）
異様な体躯：体が膨れ上がったように、過剰なまでの筋肉を付け、体格も巨躯となる。堕落は近い。
殺戮鬼：民に触れれば、もう貴卿は衝動を抑えきれない。引き裂き殺し、血を啜ってしまう。民に近づいてはならない……。
獣頭：頭が狼や山羊、蝙蝠そのものとなる。遠目にもわかる異常であり、人狼と間違えられて討たれる騎士もいるという。
原初の鎧：体の多くが硬い漆黒の塊となり、禍々しい怪物のような体となる。顔または顔の半分は、元の形を維持できるだろう。
夜鳥の翼：背中に黒い鳥の翼が生える。ストリガともはや変わらず、貴卿は常に異端と並べ揶揄されるだろう。
蝙蝠の翼：背中に蝙蝠の翼が生える。決して隠せず、限定的ながらも空を飛べる……毛の生えた黒い翼だ。
割れた肌：全身の肌がひび割れる。中からじわりと紅い光が常に漏れる。姿が美しかろうと、その様子はもはや怪物だ。
退廃の香り：甘く心蕩かす香りが体から発せられる。騎士ならばともかく、民には麻薬や媚薬のように作用するものだ。
異形の腕：悪鬼を思わせる鉤爪を備えた腕。表面はタールのような漆黒の塊で覆われ……歪んだ凶悪な形をしている。
鬼火を宿す目：目に鬼火が宿る。この異様な光は目から溢れ、炎となって常に揺らめいている。貴卿とまともに向き合う者はもういない。
悪の棘：肘や膝、拳、額、あるいはより思わぬところから、黒か赤の鋭い棘が生える。周囲の皮膚も硬質化を始める。
異様なる刻印：顔を含めた全身の肌に、紅い光による異端の如き刻印が現れる。欲望の性質によって、刻印の様相は常に変わる。
毒蛇：肩からか、あるいは尾のように、禍々しい毒蛇が生える。袖や裾から見せれば具現化させた使役獣に見えなくもないが……。
第三の目：貴卿の額に、第三の目が開く。髪やフードで隠せなくもないが……隠し通すのは難しいだろう。
拡がる鱗：貴卿の体のそこかしこが鱗に覆われ始める。ゆっくりと拡がるそれは、やがて隠しきれなくなるだろう。
獣の唸り：貴卿の内から、獣の唸り声が聞こえるようになる。やがては獣臭も漂い始めるだろう。内は既に獣となりつつある。
変身：なぜこのような……貴卿は性別、外見年齢、体格のいずれか、時にはすべてが大きく変わってしまう。
山羊の角：頭から二本の、ねじれた長い山羊角が生えてくる。この角は、具現化などによって隠したりはできない。
牙の咢：歯が全て犬歯となり、口自体も大きく開くようになる。笑顔も、威嚇しているかのように見えるだろう。
竜の翼：背中に鱗で覆われた頑強な翼が生える。滑空程度はできるだろう。幸いにも竜翼は……吉兆と見る騎士も多い。
山羊の脚：衣装で何とか隠せるのが救いだが……その脚は黒山羊のものとなってしまった。見た者は嫌悪に眉をひそめるだろう。
狼の耳と尾：貴卿の頭に狼の耳が、尻に狼の尾が現れる。これらは獣のように、貴卿の感情に合わせて動く。
闇溢れの目：目の、白目が黒くなる。さらに目からは闇がにじみ出しあふれ、黒い炎の如く揺らめく。
青ざめた肌：死体のように冷たく、また異様に青ざめた肌となる。一見して貴卿の堕落は明らかだ。
発作的変身：【渇き】が増えるごとに貴卿の姿は一瞬だけ、特定の堕落者の姿に変身してしまう。おぞましき末路の暗示なのだ。
具現化不安定：具現化が不安定となる。貴卿が具現化するものは全て歪み、優雅さを失った異形となるだろう。
異様に光る眼：目が鬼火のように光っている。それは闇におぞましい尾を引いて、無意識の内に獲物を狙うだろう。
目立つ犬歯：八重歯と言うには長く、鋭すぎる犬歯。口づけの折、ともすれば相手の肌を傷つけてしまいそうになる……。
魔女爪：手足の爪が、異様な鉤爪となる。具現化でごまかすこともできない……籠手などを常に着けておけば可能だろうか。
加減困難：貴卿の力は制御が難しくなる。ふとした拍子に、家具を壊したり民を傷つけてしまうだろう。
長い舌：貴卿の舌は、己の目まで届くほどに長くなる。幸いにも伸ばして見せない限りは、外部からわかるまいが……。
不埒な口づけ：口づけのごとに、貴卿を異様な劣情が襲う。その作法は浅ましく、淫らがましいものとなってしまうだろう。
堕落の影：紅い月光が照らす貴卿の影は、もはや堕落者のものだ。初対面ならともかく、長く気づかれずには……いられまい。
月光欲：屋根があると不快感を感じる。なるべく月光を浴びられる場所でいたい。宮廷でも庭園やバルコニーを好むだろう。
幸運：自覚するような兆しはないようだが……己を戒めねばなるまい。貴卿は今や堕落の瀬戸際にいると自覚せよ。
},},

    'BT' => {
      :name => "絆内容決定表：ルージュ／ノワール",
      :type => '1D6',
      :table => %w{
憐(Pity)　相手を憐れみ、慈しむ。／侮(Contempt)　相手を侮り、軽蔑する。
友(Friend)　相手に友情を持つ。／妬(Jealousy)　相手を羨望し、妬む。
信(Trust)　相手を信頼する。／欲(Desire)　相手を欲し、我がものにしたいと思う。
恋(Love)　相手に恋し、愛する。／怒(Anger)　相手に怒りを感じる。
敬(Respect)　相手の実力や精神を敬う。／殺(Kill)　相手に殺意を持ち、滅ぼそうと思う。
主(Obey)　相手を主と仰ぎ、忠誠を誓う。／仇(Vendetta)　相手を怨み、仇と狙う。
},},
  }

  setPrefixes(['DR.*', 'RT.*', 'HRT.*', 'CT\d+', "ST", "CO", "CA", "EP", "OS", "PN", "RS", "PP"] + @@tables.keys)
end
