# -*- coding: utf-8 -*-

class KillDeathBusiness_Korean < DiceBot
  setPrefixes([
    'HST',
    'ST[1-2]?',
    'DWT', 'DeathWT',
    'RWT', 'RevengeWT',
    'VWT', 'VictoryWT',
    'PWT', 'PossesionWT',
    'CWT', 'ControlWT',
    'FWT', 'FlourishWT',
    'IWT', 'IntensifyWT',
    'HWT', 'HealthWT',
    'SaWT', 'SafetyWT',
    'LWT', 'LongevityWT',
    'EWT', 'ExistWT',
    'NAME[1-3]?', 'NAME[1-3]?',
    'OSPT', 'OccultSPT',
    'FSPT', 'FamilySPT',
    'LoSPT', 'LoveSPT',
    'JSPT', 'JusticeSPT',
    'TSPT', 'TrainingSPT',
    'BSPT', 'BeamSPT',
    'CMT',
    'EST', 'sErviceST',
    'SOUL',
    'STGT',
    'HSAT[1-2]?',
    'EXT[1-4]?',
    'SKLT',
    'SKLJ',
    'ERT',
    'WKT',
    'PCDT',
    'OHT',
    'PCT1',
    'PCT2',
    'PCT3',
    'PCT4',
    'PCT5',
    'PCT6',
    'PCT7',
    'JD.*',
  ])

  def initialize
    super
    @sendMode = 2
    @sortType = 1
    @d66Type = 2
  end

  def gameName
    'Kill Death Business (한국어)'
  end

  def gameType
    "KillDeathBusiness:Korean"
  end

  def getHelpMessage
    return <<INFO_MESSAGE_TEXT
・판정
　JDx or JDx+y or JDx-y or JDx,z or JDx+y,z JDx-y,z
　（x＝난이도、y＝보정、z＝펌블도(리스크)）
・이력표 (HST、HSTx) x에 숫자(1,2)로 표를 개별 롤
・소원표 (-WT)
　죽음(DWT)、복수(RWT)、승리(VWT)、획득(PWT)、지배(CWT)、번영(FWT)
　강화(IWT)、건강(HWT)、안전(SAWT)、장생(LWT)、삶(EWT)
・만능이름표 (NAME) x에 숫자(1,2,3)로 표를 개별 롤
・서브플롯표 (-SPT)
　오컬트(OSPT)、가족(FSPT)、연애(LOSPT)、정의(JSPT)、수행(TSPT)
　웃음(BSPT)、심술쟁이(MASPT)、원한(UMSPT)、인기(POSPT)、구분(PASPT)
　돈벌이(MOSPT)、대(対)악마(ANSPT)
・씬 표 (ST)、서비스 씬 표 (EST)
・CM표 (CMT)
・소생 부작용 표 (ERT)
・일주일간 표（WKT)
・소울 방출표 (SOUL)
・범용연출표 (STGT)
・헬 스타일리스트 매도표 (HSAT、HSATx) x에 숫자(1,2)로 표를 개별 롤
・지정특기 랜덤 결정표 (SKLT)、지정특기 분야 랜덤 결정표 (SKLJ)
・엑스트라 표 (EXT、EXTx) x에 숫자(1,2,3,4)로 표를 개별 롤
・제작위원 결정표　PCDT/실제 어떠했는가 표　OHT
・태스크 표　헬 라이온　PCT1/헬 크로우　PCT2/헬 스네이크　PCT3/
　헬 드래곤　PCT4/헬 플라이　PCT5/헬 갓　PCT6/헬 베어　PCT7
・D66 다이스 지원
INFO_MESSAGE_TEXT
  end

  # 게임별 성공도 판정(2D6)
  def check_2D6(total_n, dice_n, signOfInequality, diff, dice_cnt, dice_max, n1, n_max)

    debug("total_n, dice_n, signOfInequality, diff, dice_cnt, dice_max, n1, n_max", total_n, dice_n, signOfInequality, diff, dice_cnt, dice_max, n1, n_max)

    return '' unless(signOfInequality == ">=")

    output =
      if(dice_n <= 2)
        " ＞ 펌블(판정 실패.【시청률】이 20%감소)"
      elsif(dice_n >= 12)
        " ＞ 스페셜(판정 성공.【시청률】이 10%증가)"
      elsif(total_n >= diff)
        " ＞ 성공"
      else
        " ＞ 실패"
      end

    return output
  end

  def rollDiceCommand(command)

    debug("rollDiceCommand command", command)

    #판정체크는 먼저 처리
    case command
    when @@judogeDiceReg
      result = judgeDice(command)
      text = "판정#{result}"
      return text
    end

    #판정 이외라면 표 커멘드의 처리를 한다
    return rollTableCommand(command)
  end

  @@judogeDiceReg = /(^|\s)JD(\d+)([\+\-]\d+)?(,(\d+))?($|\s)/i

  def judgeDice(command)

    unless(@@judogeDiceReg === command)
      return '1'
    end

    target = $2.to_i
    modify = $3.to_i
    fumble = $5.to_i

    result = ""

    if(target > 12 )
      result  += "【#{command}】 ＞ 난이도가 12이상은 스페셜만 성공.\n"
      target = 12
    end

    if(target < 5 )
      result  += "【#{command}】 ＞ 난이도의 최저치는 5.\n"
      target = 5
    end

    if( fumble < 2 )
      fumble = 2
    elsif(fumble > 11 )
      result  += "【#{command}】 ＞ 스페셜을 내면 반드시 성공이므로, 펌블치은 11으로 한다.\n"
      fumble = 11
    end

    number, diceText, = roll(2, 6)

    result += "【난이도#{target}、보정#{modify}、펌블치#{fumble}】 ＞ 주사위 눈(#{diceText}) ＞ "

    if(number == 2 )
      result += "주사위 눈이 2이므로 펌블!(판정 실패.【시청률】이 20%감소)"
    elsif(number == 12)
      result += "주사위 눈이 12이므로 스페셜!(판정 성공.【시청률】이 10%증가)"
    elsif(number <= fumble)
      result += "주사위 눈이	  펌블율 이하이므로 펌블!(판정 실패.【시청률】이 20%감소)"
    else
       number += modify
       if(number < target)
         result += "달성치#{number}、난이도미만이므로 판정 실패!"
       else
         result += "달성치#{number}、난이도이상이므로 판정 성공!"
       end
    end

    return result
  end

  def rollTableCommand(command)

    tableName = ""
    result = ""

	case command

    when "HST"
      tableName, result, number = getHistoryTableResult

    when /^ST(\d)?$/
	  #씬표
      type = $1.to_i

	  tableName, result, number = getSceneTableResult(type)

    when /^.+WT$/i
	  #소원표
      tableName, result, number = getWishTableResult(command)

    when /^NAME(\d)?$/
	  #만능이름표
      type = $1.to_i
      tableName, result, number = getNameTableResult(type)

    when /^.+SPT$/i
	  #서브플롯표
      tableName, result, number = getSubprotTableResult(command)

    when "CMT"
	  #CM표
      tableName, result, number = getCmTableResult()

    when "ERT"
	  #소생 부작용 표
      tableName, result, number = getErTableResult()

    when "WKT"
	  #일주일간 표
      tableName, result, number = getWKTableResult()

    when /^EST$/i, /^sErviceST$/i
      tableName, result, number = getServiceSceneTableResult()

    when "SOUL"
      tableName, result, number = getSoulTableResult()

    when "STGT"
      tableName, result, number = getSceneTelopGenericTableResult()

    when /^HSAT(\d)?$/
      type = $1.to_i
      tableName, result, number = getHairStylistAbuseTableResult(type)

    when /^EXT(\d)?$/
      type = $1.to_i
      tableName, result, number = getExtraTableResult(type)

    when /^SKL(T|J)$/
	  type = $1
      tableName, result, number = getSkillTableResult(type)

    when "PCDT"
	  #제작위원 결정표
      tableName, result, number = getpcTableResult()
    when "OHT"
	  #실제 어떠했는가 표
      tableName, result, number = getohTableResult()
    when "PCT1"
	  #헬 스타일리스트 표
      tableName, result, number = getplTableResult()
    when "PCT2"
	  #헬 크로우 태스크 표
      tableName, result, number = getprTableResult()
    when "PCT3"
	  #헬 스네이크 태스크 표
      tableName, result, number = getpnTableResult()
    when "PCT4"
	  #헬 드래곤 태스크 표
      tableName, result, number = getpdTableResult()
    when "PCT5"
	  #헬 플라이 태스크 표
      tableName, result, number = getpfTableResult()
    when "PCT6"
	  #헬 갓 태스크 표
      tableName, result, number = getpgTableResult()
    when "PCT7"
	  #헬 베어 태스크 표
      tableName, result, number = getpbTableResult()

    else
      debug("rollDiceCommand commandNOT matched -> command:", command)
      return ""
	end

    if( result.empty? )
      return ""
    end

	text = "#{tableName}(#{number})：#{result}"
	return text
  end

  def getHistoryTableResult
    tableName = "이력표"

    table = [
             '마술
당신은 악마와의 접촉을 위해 연구를 쌓아온 오컬티스트, 또는 공포의 탐색자다. 굉장한 고난과 노력을 통해,드디어 나타난 악마는, 소원을 들어주는 대신 방송에나가라, 다고 말했다. 당신은 조금 어이없어 하면서도,애당초 악마를 찾던 이유인 소원을 이루기 위해, 그들이 주최하는 방송에 출연할 것을 승낙했다.',
             '천재
당신은 어떤 분야에서의 천재였다. 어떤 천재였는지는,캐릭터 작성을 하는 중에 생각해서 정해도 되고, 전혀정하지 않아도 상관없다. 아무튼, 천재가 방송에 등장하면 캐릭터가 살아 재밌겠다고 생각한 악마는, 당신의눈앞에 방송의 출연교섭을 위해 나타났다. 당신은 그이야기에 따라, 혼의 회수인이 되기로 했다.',
             '광기
당신은 머리가 이상하다. 그렇기 때문에, 주변 사회에잘 적응하지 못한 인생을 보냈다. 악마는 거기에 출현해, 사회를 당신에게 맞춰 바꿔주는 것이 가능하다 말했다. 단 사람을 죽이는 게임 방송에서 우승해야하는모양이다. 그러나, 당신은 머리가 이상하므로, 사람을죽이는 점은 신경 쓰지 않았다. 아무튼 이기면 행복해지는 것이다. 힘내자.',
             '절망
당신은 어떠한 실패를 통해 실의의 밑바닥에 있다. 합법, 또는 위법인 약물에 취해, 생활은 점점 피폐해지고,슬슬 죽을 거라 생각했을 때 악마가 나타났다. 그녀석의 말에 따르면, 어떤 리얼리티 방송에서 이기면, 이상황에서 빠져나갈 수 있다고 한다. 더 이상 잃을 것은없는 당신은, 그 이야기에 따라, 방송에 출연하기로 했다.',
             '선달
당신은 과거에 『킬 데스 비즈니스』의 승리자와 아는사이다. 갑자기 이유도 없이 큰 성공을 거둔 지인에게사정을 물으니, 바로 악마가 주최하는 살인 게임 방송에 대해 알려줬다. 당신은 소원을 이루기 위해, 그 지인을 통해 악마에게 추천해 달라 하여, 방송에 참가하게 됐다. 지인은, 분명 방송을 보고 있겠지.',
             '친구
당신은 평온한 인생을 보내고 있었으나, 어느 날 눈앞에 악마가 나타났다. 그 녀석은 방송 출연의 엔트리 시트를 가지고 있어서, 거기에는 당신의 이름이 적혀 있었다. 친구가 멋대로 당신의 이름으로 방송에 엔트리시트를 보낸 것이다! 당신은 그대로 납치되어, 억지로소원을 정하게 되어, 악취미인 방송에서 폭력을 휘두르게 됐다.',
            ]
    result, number = get_table_by_1d6(table)

    return tableName, result, number
  end

  def getSceneTableResult(type)
    debug("getSceneTableResult type", type)

    tableName = "씬표"

    sceneTable1 = [
                   [11, '당신이 근거로 삼고 있는'],
                   [12, '당신의 사진으로 벽이 메워진'],
                   [13, '아날로그TV가 산처럼 쌓인'],
                   [14, '쓰레기가 뒤덮은'],
                   [15, '헬'],
                   [16, '밀어닥친 모래로 뒤덮인'],
                   [22, '가치 있는 것으로 호화롭게 장식된'],
                   [23, '시계장치의 전자의 요새로 변한'],
                   [24, '기묘한 오브제로 가득한'],
                   [25, '거대한 구멍의 가장자리에 위태롭게 걸친'],
                   [26, '극채색의 벽화가 그려진'],
                   [33, '사라질 듯이 명멸하는 빛이 비추는'],
                   [34, '창백하게 뭐든지 얼어붙은'],
                   [35, '붉은 피와 철의 냄새로 가득 찬'],
                   [36, '태양이 강렬한 빛을 뿌리는'],
                   [44, '새가 무수히 모여 있는'],
                   [45, '타오르는 불꽃이 비추는'],
                   [46, '하얀 안개로 한치 앞도 보이지 않는'],
                   [55, '썩은 시체가 쌓여있는'],
                   [56, '유독한 오수가 잔뜩 모여 있는'],
                   [66, '약탈을 당해 텅 빈'],
                  ]
    sceneTable2 = [
                   [11, '오피스'],
                   [12, '제단'],
                   [13, '스튜디오'],
                   [14, '창고'],
                   [15, '버스 정류장'],
                   [16, '감옥'],
                   [22, '암흑'],
                   [23, '역'],
                   [24, '옥상'],
                   [25, '하수도'],
                   [26, '학교'],
                   [33, '연구실'],
                   [34, '차 안'],
                   [35, '삼림'],
                   [36, '정원'],
                   [44, '가게'],
                   [45, '농원'],
                   [46, '폐허'],
                   [55, '묘지'],
                   [56, '민가'],
                   [66, '들판'],
                  ]

    result = ''
    number = 0

    case type
    when 1
      result, number = get_table_by_d66_swap(sceneTable1)
    when 2
      result, number = get_table_by_d66_swap(sceneTable2)
    else
      result1, num1 = get_table_by_d66_swap(sceneTable1)
      result2, num2 = get_table_by_d66_swap(sceneTable2)
      result = "#{result1} #{result2}"
      number = "#{num1},#{num2}"
    end

    return tableName, result, number
  end

  def getWishTableResult(command)
    debug("getWishTableResult command", command)

    tableName = ''
    table = []

    case command
    when /^DWT$/i, /^DeathWT$/i
      tableName = "소원표「죽음」"
      table = [
               '지금의 아름다움을 유지한 채 죽고 싶다.',
               '재밌는 방법으로 죽어보고 싶다.',
               '자살할 용기가 없어서 죽고 싶다.',
               '죽으면 어떻게 될지 보고 싶다.',
               '너무나 싫은 일이 있어서 죽고 싶다.',
               '불로불사가 질려서 죽고 싶다.',
              ]
    when /^RWT$/i, /^RevengeWT$/i
      tableName = "소원표「복수」"
      table = [
               '가족을 죽인 범인에게 복수하고 싶다.',
               '자신보다 행복한 이웃에게 복수하고 싶다.',
               '자신을 배반한 친구에게 복수하고 싶다.',
               '자신을 버린 애인에게 복수하고 싶다.',
               '소중한 것을 빼앗은 범인에게 복수하고 싶다.',
               '처참한 패배를 안겨준 라이벌에게 복수하고 싶다.',
              ]
    when /^VWT$/i, /^VictoryWT$/i
      tableName = "소원표「승리」"
      table = [
               '이전에 졌던 라이벌에게 이기고 싶다.',
               '쓰러트리고 싶은 유명인(고인)이 있다.',
               '쓰러트리고 싶은 유명인이 있다.',
               '특정 업계에서 제일의 실력자가 되고 싶다.',
               '특정 게임에서 최강의 플레이어가 되고 싶다.',
               '특정 테스트에서 1위를 하고 싶다.',
              ]
    when /^PWT$/i, /^PossesionWT$/i
      tableName = "소원표「획득」"
      table = [
               '원하는 아이템이 있다.',
               '원하는 생물이 있다.',
               '원하는 관계가 있다.',
               '원하는 친족이 있다.',
               '원하는 지식이 있다.',
               '원하는 부동산이 있다.',
              ]
    when /^CWT$/i, /^ControlWT$/i
      tableName = "소원표「지배」"
      table = [
               '지배하고 싶은 인물집단이 있다.',
               '지배하고 싶은 지역이 있다.',
               '세계를 정복하고 싶다.',
               '선거에서 이기고 싶다.',
               '굴욕을 주고 싶은 사람이 있다.',
               '어리광쟁이인 친척을 마음대로 하고 싶다.',
              ]
    when /^FWT$/i, /^FlourishWT$/i
      tableName = "소원표「번영」"
      table = [
               '몰락한 일족을 부흥시키고 싶다.',
               '지역 경제를 활성화시키고 싶다.',
               '취미인 장르에서 사람을 모으고 싶다.',
               '절멸한 생물을 부활시키고 싶다.',
               '동물이나 식물을 절멸에서 구하고 싶다.',
               '아무튼 많은 돈을 원한다.',
              ]
    when /^IWT$/i, /^IntensifyWT$/i
      tableName = "소원표「강화」"
      table = [
               '세계최강이 되고 싶다.',
               '세계에서 가장 아름다워지고 싶다.',
               '세계에서 가장 현명해지고 싶다.',
               '세계에서 가장 행복해지고 싶다.',
               '세계에서 가장 터프해지고 싶다.',
               '세계에서 가장 빨라지고 싶다.',
              ]
    when /^HWT$/i, /^HealthWT$/i
      tableName = "소원표「건강」"
      table = [
               '완성시키고 싶은 약물이 있다.',
               '절멸시키고 싶은 병원균이 있다.',
               '난치병에서 회복시키고 싶은 친척이 있다.',
               '난치병에서 회복하고 싶다.',
               '병으로 죽은 사람을 부활시키고 싶다.',
               '병에 걸리지 않는 몸을 원한다.',
              ]
    when /^SaWT$/i, /^SafetyWT$/i
      tableName = "소원표「안전」"
      table = [
               '평온한 생활을 보내고 싶다.',
               '평온하게 지내길 바라는 사람이 있다.',
               '끝내고 싶은 분쟁이 있다.',
               '사고로 죽은 사람을 부활시키고 싶다.',
               '사고를 겪지 않는 능력을 원한다.',
               '다가오는 세계의 위기를 회피하고 싶다.',
              ]
    when /^LWT$/i, /^LongevityWT$/i
      tableName = "소원표「장수」"
      table = [
               '지금의 아름다움을 유지하고 싶다.',
               '지금의 강함을 유지하고 싶다.',
               '아이가 어른이 되기까지 죽지 않는다.',
               '불로의 몸을 원한다.',
               '치매에 걸리고 싶지 않다.',
               '손자의 얼굴을 보고 싶다.',
              ]
    when /^EWT$/i, /^ExistWT$/i
      tableName = "소원표「삶」"
      table = [
               '살아 있다는 실감을 원한다.',
               '지금과는 다른 인생을 살고 싶다.',
               '죽어버렸으니 부활하고 싶다.',
               '죽은 사람을 되살리고 싶다.',
               '불사의 몸을 원한다.',
               '이상의 인생을 손에 넣고 싶다.',
              ]
    end

    result, number = get_table_by_1d6(table)
    debug("getWishTableResult result", result)

    return tableName, result, number
  end

  def getNameTableResult(type)
    tableName = "만능이름표"

    nameTable1 = [
                  [11, '아날로그'],
                  [12, '이지'],
                  [13, '이터널'],
                  [14, '일렉트로'],
                  [15, '오토'],
                  [16, '킹'],
                  [22, '쿨'],
                  [23, '그림'],
                  [24, '그레이트'],
                  [25, '슈퍼'],
                  [26, '스푸크'],
                  [33, '스마트'],
                  [34, '드레드'],
                  [35, '하이테크'],
                  [36, '헌터즈'],
                  [44, '블랙'],
                  [45, '헬'],
                  [46, '매직'],
                  [55, '메카니컬'],
                  [56, '러브'],
                  [66, '레드'],
                 ]
    nameTable2 = [
                  [11, 'X'],
                  [12, '어비스'],
                  [13, '그레이브'],
                  [14, '골드'],
                  [15, '고딕'],
                  [16, '시노비'],
                  [22, '스페셜'],
                  [23, '데몬'],
                  [24, '디지털'],
                  [25, '데스'],
                  [26, '데빌'],
                  [33, '나이트메어'],
                  [34, '발록'],
                  [35, '블러드'],
                  [36, '브레이크'],
                  [44, '블레이즈'],
                  [45, '프로스트'],
                  [46, '메가'],
                  [55, '메카나이즈드'],
                  [56, '림보'],
                  [66, '로보'],
                 ]
    nameTable3 = [
                  [11, '암'],
                  [12, '아이'],
                  [13, '이어'],
                  [14, '윙'],
                  [15, '엔진'],
                  [16, '가드'],
                  [22, '건'],
                  [23, '기어'],
                  [24, '캐논'],
                  [25, '서킷'],
                  [26, '스킨'],
                  [33, '소드'],
                  [34, '터빈'],
                  [35, '하트'],
                  [36, '핸드'],
                  [44, '빔'],
                  [45, '플레이트'],
                  [46, '헤어'],
                  [55, '메카니즘'],
                  [56, '모터'],
                  [66, '레그'],
                 ]

    result = ''
    number = 0

    case type
    when 1
      result, number = get_table_by_d66_swap(nameTable1)
    when 2
      result, number = get_table_by_d66_swap(nameTable2)
    when 3
      result, number = get_table_by_d66_swap(nameTable3)
    else
      result1, num1 = get_table_by_d66_swap(nameTable1)
      result2, num2 = get_table_by_d66_swap(nameTable2)
      result3, num3 = get_table_by_d66_swap(nameTable3)
      result = "#{result1}#{result2}#{result3}"
      number = "#{num1},#{num2},#{num3}"
    end

    return tableName, result, number
  end

  def getSubprotTableResult(command)
    tableName = ''
    table = []

    case command
    when /^OSPT$/i, /^OccultSPT$/i
      tableName = "서브플롯표「오컬트」"
      table = [
               '『죽음의 탐구』
회수인으로부터 한 명을 골라, 상대가 「직업」으로 습득하고 있는 특기를 지정특기로판정한다. 판정에 성공하면, 방송에 의한 빈번한 죽음이 상대를 어떻게 변화시키는지 관찰하고싶어진다. 상대에 대한 관계를, 속성 「연구대상」으로 획득한다. 【심도】는 1이 된다.
클리어조건 : 상대가, 당신이 등장하고 있는 신에서 2번사망한다. 사망에 따라 상대가 어떻게 변하는지, 제멋대로 이러쿵저러쿵 말해도 되고, 아무것도 말하지 않고 다 안 것 같은 표정을 해도 된다.
보수：【시청률】10%증가、400【소울】',
               '『내방자』
캐릭터로부터 한 명을 골라, 상대가 「의장」으로 습득하고 있는 특기를 지정특기로 판정한다. 판정에 성공하면, 상대가 입고 있는 것은 교묘한 위장이며, 그 정체는 우주인이란 것을 깨닫는다. 상대에 대한 관계를, 속성 「우주인」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 2이상으로 한다. 그러면, 상대는 자신의 진짜 모습을 알려준다. 정말로 우주인인지는 상대의 마음이고, 딱히 어느 쪽이든 상관없다. 적당히 기뻐하든 실망하든 할 것.
보수：【시청률】10%증가、200【소울】',
               '『기억상실』
캐릭터로부터 한 명을 골라, 상대가 「소원」으로 습득하고 있는 특기를 지정특기로 판정한다. 판정에 성공하면, 그 순간, 상대가 무언가를 알고 있다는 확신만을 남기고, 다른 모든 개인적인 기억을 잃는다. 상대에 대한 관계를, 속성 「지인」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 3으로 한다. 그러면, 상대는 당신이 어떤 인물이었는지 생각나게 해준다. 직접 알려줘도 좋고, 별의미 없는 행동이 계기가 될지도 모른다.
보수：【시청률】10%증가、상대가 회수인이라면400【소울】、상대가 NPC라면600【소울】',
               '『비밀결사』
캐릭터로부터 한 명을 골라, 상대가 「동작」으로 습득하고 있는 특기를 지정특기로 판정한다. 판정에 성공하면, 상대가 무의식적으로 한 동작이, 당신이 소속된 비밀결사의 멤버라는 것을 밝히는 사인이라고 파악한다. 상대에 대한 관계를, 속성 「멤버」로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 2이상으로 한다. 상대는 정말로 멤버일지도 모르고, 당신의 착각일지도 모른다. 어느 쪽이든, 탐색하며 가까워지는 과정에서, 상대를 잘 알게 된다.
보수：【시청률】10%증가、200【소울】',
               '『살인귀』
캐릭터로부터 한 명을 골라, 상대가 「정동」으로 습득하고 있는 특기를 지정특기로 판정한다. 판정에 성공하면, 상대가 유명한 연쇄살인의 범인이라고 확신한다. 얼굴이나 성별은 바꾼 것일지도 모른다. 상대에 대한 관계를, 속성 「살인귀」로 획득한다. 【심도】는 1이 된다.
클리어조건 : 연쇄살인귀를 용서할 수 없다면, 상대를 전투로 한 번 쓰러트린다. 또는, 연쇄살인귀의 팬이라면, 상대와 함께「서비스」신을 성공시킨다. 어느 쪽이든, 상대가 정말 연쇄살인을 했는지는 분명하지 않다.
보수：【시청률】10%증가、클리어조건 달성 시의 상대에 대한 관계【심도】× 200【소울】',
               '『마법사』
캐릭터로부터 한 명을 골라, 상대가 「소도구」으로 습득하고 있는 특기를 지정특기로 판정한다. 판정에 성공하면, 상대는 애초에 진짜로 마법을 사용할 수 있는 인간이며, 가지고 있는 도구는 고명한 매직 아이템이라고 확신한다. 상대에 대한 관계를, 속성 「마법사」으로획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 2이상으로
하고, 또한 상대의 당신에 대한 관계의
【심도】를 1이상으로 한다. 정체가 드러난 상대는 마법을 가르쳐주고, 당신은 만족한다. 덧붙여, 이 마법은 효과가 애매하고 확실하지 앟은 것을 적당히 설정한다.
보수：【시청률】10%증가、당신과 상대 모두 300【소울】',
              ]
    when /^FSPT$/i, /^FamilySPT$/i
      tableName = "서브플롯표「가족」"
      table = [
               '『설득』
캐릭터로부터 한 명을 골라, 상대가 「소도구」로 습득하고 있는 특기를 지정특기로 판정한다. 판정에 성공하면, 상대가 가지고 있는 것은 당신이 선물한 것이며, 옛 친구가 악취미인 쇼에 나왔단 것을 알 수 있다. 그만두도록 설득하지 않으면 상대에 대한 관계를, 속성 「옛 친구」로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 3으로 한다. 그러면, 설득을 할 만큼 한 당신은 만족한다. 상대는 분명 그만두지 않는다. 또는, 선택한 「옛 친구」가 사망하거나(회수인이라면)지옥에 떨어진다. 그러면, 아무래도 포기하게 된다.
보수：【시청률】10%증가、400【소울】',
               '『이별』
캐릭터로부터 한 명을 골라, 상대가 「동작」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대가 가족 특유의 특징을 가지고 있단 것을 알고, 상당히 예전에 이별한 가족이라 확신한다. 상대에 대한 관계를, 속성 「이별」로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 3으로 한다.
그러면, 상대는 당신을 기억해낸다. 전혀 다른 환경에서 자랐을지도 모르지만, 당신들의 뿌리는 같다.
속사정이나 이별한 상황은 좋을 대로 설정해도 되고, 딱히 설정하지 않아도 된다.
보수：【시청률】10%증가、상대가 회수인이라면 400【소울】상대가 NPC라면 600【소울】',
               '『가족자랑』
회수인으로부터 한 명을 골라, 상대가 「직업」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대의 직업을 듣고, 같은 분야에서 상대보다 성공한 가족의 이야기를 들려준다.
상대에 대한 관계를, 속성 「자랑함」으로 획득한다. 【심도】는 1이 된다.
클리어조건：세 사람의 캐릭터에 대해 【심도】1이상의 관계를 획득하고, 모든 속성을 「자랑함」으로 한다.
이미 「자랑함」인 캐릭터는, 죽든지 없어져도 괜찮다.
상당히 싫어하는 모양이나, 뭐어 상관없다.
보수：【시청률】10%증가、400【소울】',
               '『버려진 고양이』
회수인으로부터 한 명을 골라, 상대가 「정동」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 품에서 아기 고양이가 나온다. 이것은 상대가 예전에 버린 고양이인 걸까, 당신은 그것을 용서할 수 없다.
상대에 대한 관계를, 속성 「고양이의 주인」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에게 고양이를 기르게 하기 위해, 전투로 한 번 쓰러트린다.
그러면, 상대는 고양이를 기를 것에 동의한다.
고양이의 상세한 스펙을 정해도 되고, 정하지 않아도 된다.
그리고, 동물의 종류도 사실은 자유다.
보수：【시청률】10%증가、클리어조건  달성 시의 상대에 대한 관계의【深度】× 200【소울】',
               '『의가족』
회수인으로부터 한 명을 골라, 상대가 「의장」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 의장의 경향에서, 상대가 의가족인 것이 판명된다. 이런 곳에서 뭘 하고 있는거야.
상대에 대한 관계를, 속성 「의가족」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 2이상으로 한다.
그러면, 상대는 어째서 방송에 나오게 됐는지, 그 때 어떤 상황이나 기분이었는지를 말한다.
또는, 상대가 지옥에 떨어지면, 연이 끊어져서 클리어다.
보수：【시청률】10%증가、200【소울】',
               '『고독한 사람』
캐릭터로부터 한 명을 골라, 상대가 「소원」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대가 고독을 견디고 있는 외로운 인물이란 것을 확신한다. 당신이 상대를 구하는 것이다.
상대에 대한 관계를, 속성 「고독한 사람」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 2이상으로 한다.
그러면, 상대는 변할……지도 모르고, 아무것도 변하지 않을지도 모른다. 그저, 당신은, 상대의 고독이 조금 치유된 것을 알 수 있다.
보수：【시청률】10%증가、200【소울】',
              ]
    when /^LoSPT$/i, /^LoveSPT$/i
      tableName = "서브플롯표「연애」"
      table = [
               '『금단의 사랑』
표적이 「의장」으로 습득하고 있는 특기를 지정특기로 판정한다. 표적이 이미 사망한 경우, 서브플롯표를 다시 굴린다.
판정에 성공하면, 표적에게 첫눈에 반하거나, 표적이 전부터 좋아했던 사람이란 것이 판명된다. 서두르지 않으면 죽어버린다.
상대에 대한 관계를, 속성 「좋아하는 사람」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 3으로 한다.
그러면, 표적은 당신의 마음에 대해 어떠한 대답을 돌려준다. 결과는 아무튼, 좋은 경험이 될 것이다.
또는, 상대에 대한 관계의 【심도】가 2이상인 상태에서, 상대가 사망해도 좋다. 통곡하자.
보수：【시청률】10%증가、400【소울】',
               '『방송 내 데이트』
회수인으로부터 한 명을 골라, 상대가 「정동」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대와 당신은 방송 개시 전부터 사귀고 있었다. 수록을 겸해 사이를 진전시켜도 좋을 것이다.
상대에 대한 관계를, 속성 「연인」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 2이상으로 하고, 상대로부터 당신에 대한 관계의 【심도】도 2이상으로 한다.
그러면, 꽤 좋은 느낌으로 관계가 발전한다. 방송 종료 시에 시청자에게 알릴 일이 생길지도 모른다.
보수：【시청률】10%증가、당신과 상대 모두 400【소울】',
               '『고민 상담』
캐릭터로부터 한 명을 골라, 상대가 「직업」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대의 직업적 견지에서 연애에 관한 어드바이스를 받을지도 모른다고 깨닫는다.
상대에 대한 관계를, 속성 「상담역」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 3으로 한다.
연애에 관한 어드바이스를 받은 기분이 든다. 상대가 그런 의도로 말했는지는 관계없다.
최후의 관계의 속성을 「좋아하는 사람」으로 하면 그럴싸해진다.
보수：【시청률】10%증가、상대가 회수인이라면400【소울】상대가 NPC라면600【소울】',
               '『인기획득』
캐릭터로부터 한 명을 골라, 상대가 「동작」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대와 당신이 함께하면 대단히 좋은 장면이 나와 시청자에게 먹힐 것이라 확신한다. 끌어들여 서비스 신을 성공하면 헬 시청자에게 사랑받겠지.
상대에 대한 관계를, 속성 「파트너」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대와 함께 서비스 신을 성공시킨다.
놀랍게도, 헬 시청자는 정말로 콤비를 마음에 들어 한다. 단, 한 번 더 한다고, 먹힐지는 모른다.
보수：【시청률】10%증가、200【소울】',
               '『선물』
캐릭터로부터 한 명을 골라, 상대가 「소원」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대의 소원을 왠지 알 수 있다. 원하는 것을 알고 있으니, 주면 좋아할지도 모른다. 뭘 원하는지, 상대에게 실제로 물을 필요는 없다.
상대에 대한 관계를, 속성 「원함」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 2이상으로 한다.
그러면, 상대는 당신의 선물을 받는다. 무엇을 주는지는 적당히 정해도 되나, 고민되면 헬CM표를 사용할 것.
보수：【시청률】10%증가、200【소울】',
               '『수제요리』
캐릭터로부터 한 명을 골라, 상대가 「소도구」로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대가 가지고 있는 것이 힌트가 돼서, 엄청난 요리를 떠올린다. 어쩌면 재료로 할 수 있을지도 모른다.
상대에 대한 관계를, 속성 「시식역」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에게 전투를 걸어, 킬러트릭으로 쓰러트린다.
이때, 킬러트릭의 이름은 생각한 요리의 이름으로 변경된다.
고민되면 만능 명명표를 사용할 것.
보수：【시청률】10%증가、500【소울】',
              ]
    when /^JSPT$/i, /^JusticeSPT$/i
      tableName = "서브플롯표「정의」"
      table = [
               '『죽일 수 없는 표적』
표적이 「정동」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대가 당신의 정의에 비추어 볼 때 허물없는 인물이란 것을 알 수 있다(어쩌면, 실은 대단히 선량한 인물이었단 것을 안다). 이 사람을 죽일 수는 없다.
상대에 대한 관계를, 속성 「호위대상」으로 획득한다. 【심도】는 1이 된다.
클리어조건：결과 페이즈까지 상대를 생존시킨다.
어째서 아군이 됐는지, 어째서 아군으로 삼았는지, 상대에게 말해주면 기뻐할지도 모른다.
보수：【시청률】10%증가、700【소울】',
               '『친구의 복수』
캐릭터로부터 한 명을 골라, 상대가 「소도구」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대가 몸에 지니고 있는 장신구가, 당신의 친구를 살해한 범인의 특징과 일치하는 것을 안다.
상대에 대한 관계를, 속성 「범인」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 2이상으로 한다.
그러면, 상대는 자백한다.
그 후에 전투를 걸어, 한 번 상대를 쓰러트려라.
죽은 자는 돌아오지 않으나, 당신은 만족한다.
보수：【시청률】10%증가、400【소울】',
               '『호신』
캐릭터로부터 한 명을 골라, 상대가 「소원」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대가 한때 당신에게 단죄 받은 적이 있어, 그것을 원망하고 있는 것을 알게 된다.
상대에 대한 관계를, 속성 「복수자」로 획득한다. 【심도】는 1이 된다.
클리어조건：결과페이즈까지 상대에게 살해당하지 않는다.
그러면, 어떻게든 살아남……지만, 다음은 어떻게 될는지.
보수：【시청률】10%증가、상대가 회수인이라면500【소울】상대가 NPC라면 100【소울】',
               '『심취』
회수인으로부터 한 명을 골라, 상대가 「동작」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대를 첫 인상부터 선인이라 확신하고, 다양한 논리를 더해 상대의 소원을 정의라고 믿는다.
상대에 대한 관계를, 속성 「선인」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 2이상으로 한다.
그러면, 상대의 소원의 본질을 이해하고, 깨어나거나, 충실한 협력자가 될 것을 약속하거나 한다.
보수：【시청률】10%증가、200【소울】',
               '『악당』
캐릭터로부터 한 명을 골라, 상대가 「직업」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대는 그 일대에서는 유명한 악의 직업인이란 것이 판명된다. 이 녀석을 내버려두면 슬퍼하는 사람이 나오겠지.
상대에 대한 관계를, 속성 「악인」으로 획득한다. 【심도】는 1이 된다.
클리어조건：악인을 용서할 수 없다면, 상대를 전투에서 한 번 쓰러트린다.
또는, 개심시키고 싶다면, 상대에 대한 관계의 【심도】를 3으로 한다.
상대가 정말로 개심했는지는 관계없이, 회심의 설득을 한 것에, 당신은 만족한다.
보수：【시청률】10%증가、클리어조건 달성 시의 상대에 대한 관계【심도】X200【소울】',
               '『사이드킥』
회수인으로부터 한 명을 골라, 상대가 「의장」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대는 자신의 파트너로써 악을 쓰러트리기에 최적인 인물이라 확신한다. 사이드킥이다.
상대에 대한 관계를, 속성 「동료」으로 획득한다. 【심도】는 1이 된다.
클리어조건：자신이 등장하고 있는 신에서, 상대가 누군가를 쓰러트리고, 그 후 당신이 죽지 않고 전투가 종료된다.
상대를 칭찬해서, 동료에 어울리는 인물이라고 선언하자.
보수：【시청률】10%증가、300【소울】',
              ]
    when /^TSPT$/i, /^TrainingSPT$/i
      tableName = "서브플롯표「수행」"
      table = [
               '『전투광』
회수인으로부터 한 명을 골라, 상대가 「동작」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 주변에 있는 강해보이는 사람들에 대해 대항심이 끓어오른다. 결국, 남는 것은 한 사람 뿐이니까, 지금부터 때려눕히는 게 좋다.
상대에 대한 관계를, 속성 「적」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대를 포함해 세 명의 캐릭터를 한 번 이상 사망시킨다.
그러면, 당신의 전투열은 가라앉는다. 이제부터 사이가 좋아질지는 미묘할지도 모른다.
보수：【시청률】10%증가、600【소울】',
               '『제자입문』
캐릭터로부터 한 명을 골라, 상대가 「직업」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 당신은, 그 직업의 세계적인 권위를 발견한다. 이유가 있어 은퇴한지 오래지만, 가르침을 받으면 이후의 인생에 도움이 될 것이다. 가르칠 마음이 있는지는 모르겠지만
상대에 대한 관계를, 속성 「스승」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 3으로 한다.
그러면, 상대는 무언가를 가르쳐준다. 또는, 더 이상 가르칠 것이 아무것도 없다고 말할 것이다.
보수：【시청률】10%증가、상대가 회수인이라면400【소울】상대가 NPC라면600【소울】',
               '『라이벌』
캐릭터로부터 한 명을 골라, 상대가 「소원」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대는 당신에게 특정 분야에서 승리한 적이 있는 인물임을 알게 된다. 녀석에게 이기지 못하면, 당신은 앞으로 나아갈 수 없다.
상대에 대한 관계를, 속성 「라이벌」로 획득한다. 【심도】는 1이 된다.
클리어조건：2사이클 이후에 상대를 한 번 쓰러트린다. 당신이 쓰러트리기까지, 상대는 다른 캐릭터에게 쓰러져서는 안 된다.
너를 쓰러트리는 것은 나로 정해져있으니까!
보수：【시청률】10%증가、상대가 회수인이라면300【소울】상대가 NPC라면500【소울】',
               '『개조』
회수인으로부터 한 명을 골라, 상대가 「의장」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대에 비해 자신의 육체가 너무 빈약한 기분이 든다. 개조해서 더욱 강해지지 않으면.
상대에 대한 관계를, 속성 「기준선」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대보다 많은 헬 사이버를 인스톨한 상태로 결과 페이즈를 맞이한다.
이때 세는 것은 개수뿐, 그리고 같은 수라면 안 된다.
보수：【시청률】10%증가、600【소울】',
               '『장비개선』
회수인으로부터 한 명을 골라, 상대가 「소도구」로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대가 가지고 있는 장비에 대한 흥미가 솟는다. 자신의 능력상승의 힌트가 될지도 모른다.
상대에 대한 관계를, 속성 「좋은 장비」로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 2이상으로 한다.
그러면, 상대는 가진 것에 대해 해설해주거나, 또는 단호하게 거절한다. 당신의 탐색은 거기서 끝나나, 경험은 남는다.
보수：【시청률】10%증가、200【소울】',
               '『특훈』
캐릭터로부터 한 명을 골라, 상대가 「정동」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대는 자신과 마찬가지로 수행부족을 통감하고 있다고 확신한다. 함께 트레이닝이다.
상대에 대한 관계를, 속성 「특훈동료」로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 2이상으로 한다.
그러면, 당신과 상대는 어떠한 성과를 느끼고, 특훈은 끝난다.
이 타이밍에서 킬러트릭의 이름을 바꾸거나 해도 된다.
보수：【시청률】10%증가、200【소울】',
              ]
    when /^BSPT$/i, /^BeamSPT$/i
      tableName = "서브플롯표「웃음」"
      table = [
               '『대 서비스』
캐릭터로부터 한 명을 골라, 상대가 「정동」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 이 살벌한 방송을 온화하게 할 수 있는 것은, 자신과 상대 밖에 없다고 확신한다.
상대에 대한 관계를, 속성 「파트너」로 획득한다. 【심도】는 1이 된다.
클리어조건：상대와 함께, 「서비스」신의 판정에 2번 성공한다.
그러면, 어쩐지 방송이 즐거워진 것 같은 느낌도 드나……변하지 않은 것 같은 기분도 든다. 당신의 싸움은 아직 계속된다.
보수：【시청률】10%증가、당신과 상대 모두 【소울】',
               '『벗기』
캐릭터로부터 한 명을 골라, 상대가 「의장」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대의 의장을 좋은 느낌으로 벗기면 먹힐 것이라 확신한다.
상대에 대한 관계를, 속성 「벗기고 싶다」로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에게 전투를 걸어, 킬러트릭으로 쓰러트린다.
이때, 상대의 의장은 산산조각으로 분쇄되어, 헬 스타일리스트가 상대를 매도하며 새로운 것을 준비하게 된다.
보수：【시청률】10%증가、500【소울】',
               '『좋은 이야기』
캐릭터로부터 한 명을 골라, 상대가 「동작」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대는 풀이 죽었는지 싫은 건지, 표정이 적다. 웃음을 돌려주지 않으면.
상대에 대한 관계를, 속성 「어두운 녀석」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 3으로 한다.
상대는 조금 웃음을 보일지도 모르고, 빛의 영향으로 그렇게 보일지도 모른다. 그러나, 당신은 확실한 성과를 느낀다.
보수：【시청률】10%증가、상대가 회수인이라면400【소울】상대가 NPC라면600【소울】',
               '『웃음전도사』
캐릭터로부터 한 명을 골라, 상대가 「소원」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 상대는 원호가 없으면 서비스 신이 힘들 것이다, 라고 직감한다.
상대에 대한 관계를, 속성 「진지한 녀석」으로 획득한다. 【심도】는 1이 된다.
클리어조건：상대가 등장하는 「서비스」신에 자신도 등장해, 카메라 밖에서 웃음소리를 내는 롤플레이를 한다.
보수：【시청률】10%증가、당신과 상대 모두 100【소울】',
               '『전투회피』
캐릭터로부터 한 명을 골라, 상대가 「직업」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 당신은 상대의 소원에 깊게 공감한다. 그 때문에, 상대를 방해하고 싶지 않게 된다.
상대에 대한 관계를, 속성 「이기게 하고픈 사람」으로 획득한다. 【심도】는 1이 된다.
클리어조건：결과 페이즈까지 직접 상대를 죽이지 않는다.
그러면, 누구도 알아채지 못하지만, 당신은 자신이 할 일을 한 게 된다.
보수：【시청률】10%증가、200【소울】',
               '『친구』
캐릭터로부터 한 명을 골라, 상대가 「소도구」으로 습득하고 있는 특기를 지정특기로 판정한다.
판정에 성공하면, 그 소도구에 대한 대화를 계기로, 상대에 대한 흥미가 싹튼다. 어쩌면 친구가 될지도 모른다.
상대에 대한 관계를, 속성 「친구」로 획득한다. 【심도】는 1이 된다.
클리어조건：상대에 대한 관계의 【심도】를 2이상으로 한다.
그러면, 당신과 상대는 우정과 같은 것으로 묶일지도 모르고, 역시 무리였다고, 당신이 포기할지도 모른다.
보수：【시청률】10%증가、200【소울】',
              ]

    end

    result, number = get_table_by_1d6(table)

    return tableName, result, number
  end

  def getCmTableResult()
    tableName = "CM표"
    table = [
             [11, '당신의 T셔츠
결정 포즈의 모습이 등에 프린트 된 T셔츠입니다. 그리고 앞면에는 뭣이 그 명대사가!'],
             [12, '당신의 액션 피규어
방송 중부터 대인기! 직인이 하나하나 혼을 담은 수제!
밤이 되면, 집어넣은 혼이 귀곡성을 냅니다.'],
             [13, '헬 레이 박스
시즌 666의 전부가 여기에……특전 영상 만재로 보내드리는, 호화 박스 세트! 예약특전은 헬군 스트랩입니다.'],
             [14, '헬 에어컨
이것만 있으면, 화재 현장이라도 기온은 어는 점 이하! 반대로 집을 태울 수도 있습니다. 온도조절기능은 유료 옵션입니다!'],
             [15, '헬 기저귀
헬 원단으로 젖음을 완벽방지, 666년 간 입더라도 걱정 없습니다! 안쪽이 어떻게 될지는 보증하지 않습니다, 헬 기저귀!
헬 서플리'],
             [16, '헬 서플리
자지 않아도 괜찮아! 컬러풀한 환각이 보인다! 하지만 메인 기능은 건강증진입니다. 헬 서플리로 건강한 생활을!'],
             [22, '헬 투어
저렴한 요금으로 지옥백경을 돌아보자! 고통 받는 인간과의 혼의 교감이 가득! 헬 투어는, 타사에서는 불가능한 체험을 약속합니다.'],
             [23, '헬 테레비
밤중에 여성이 나오나, 신경 쓰지 않아도 괜찮습니다! 아날로그 헬 테레비는 정부의 협찬을 받고 있습니다.'],
             [24, '헬 맥주
알코올 도수 666%, 헬 보리 사용. 지옥의 갈증에는 헬 맥주……차이를 아는 사람을 위해.'],
             [25, '헬 폰
의기양양한 앱등이의 콧대를 눌러버릴 수 있습니다. 네, 헬폰이라면 말이죠.'],
             [26, '헬 만쥬
수천 년의 사랑을 받고 있습니다, 지옥명과 헬만쥬. 맛있다. 헬 맛있어! 그렇지만, 먹으면 죽습니다.'],
             [33, '헬 모빌
인간을 치면 칠수록 점점 빨라져……속도는 다른 세계로. 헬 모빌이 구하기 쉬운 가격으로 신등장.'],
             [34, '헬 회화교실
이웃을 잘 속이고 싶나요? 싫은 녀석에게 속으셨나요? 헬 회화를 배우면, 어떤 상대도 격앙시킬 수 있습니다!'],
             [35, '헬 금융
금리 666%으로 안심할 수 있습니다. 헬 금융의 징수는, 대단히 신사적이라 호평입니다.'],
             [36, '헬 즙(건강음료)
지옥 변경에서 채취한 신선한 독초를, 그대로 분말로 했습니다.
이걸 마시면, 더 이상 병이나 노화 걱정은 없습니다. 건강음료 헬 즙!'],
             [44, '헬 정형외과
무보험이며 무면허나, 어떤 주문이든 받아줍니다. 헬 정형외과는, 당신이 바라는 대로의 외모를 실현합니다!'],
             [45, '헬 생명보험
곧 죽을 당신에게, 헬 생명의 안심코스. 남겨진 사람에게는 지옥이 기다리고 있습니다.'],
             [46, '헬 세제
얼마나 심한 핏자국이나 오물도 일격! 어떤 색의 옷이라도 지옥의 검정으로 물들입니다! 헬 세제, 호평 발매 중!'],
             [55, '헬 세탁기
666층식, 음속을 넘는 회전이 당신의 의장을 산산히! 무엇을 하려고 했는지 잊는다면 장관입니다. 헬 세탁기!'],
             [56, '헬 청소기
호평인 벽면분쇄기능에 더해, 올해의 모델에는 옥상흡인기능도!
헬 청소기의 스위치를 넣는 것만으로, 당신의 집은 없어집니다.'],
             [66, '헬 입욕제
몸이 끓는다! 신감각으로 화제를 부른 헬 입욕제가 드디어 등장! 사체의 처리에는 사용하지 말아주세요.'],
            ]
    result, number = get_table_by_d66_swap(table)

    return tableName, result, number
  end

  def getErTableResult()
    tableName = "소생부작용표"
    table = [
             [11, '늙다리같은 냄새가 나기 시작한다'],
             [12, '덤으로 염소가 나왔다'],
             [13, '머리색이 형광색으로 변했다'],
             [14, '목소리가 로봇처럼 변했다'],
             [15, '끝이 삼각형으로 생긴 꼬리가 생겼다'],
             [16, '얼굴에 수수께끼의 프로그레스 바가 나왔다'],
             [22, '성별이 바꼇다'],
             [23, '온몸에 모자이크가 걸렸다'],
             [24, '전체적으로 반투명이 됬다'],
             [25, '조금 썩었다'],
             [26, '피푸가 형광색으로 빛난다'],
             [33, '날개가 자랐다'],
             [34, '프로그램 참가중의 기억을 잃었다'],
             [35, '이마에 제 3의 눈이 열렸다'],
             [36, '미묘하게 뜬다'],
             [44, '병아리로 변했다'],
             [45, '불합격품의 스탬프가 찍혔다'],
             [46, '살쪗다'],
             [55, '귀가 좋아하는 동물의 것으로 바꼇다'],
             [56, '말라졌다'],
             [66, '훌륭한 뿔이 이마에 났다'],
            ]
    result, number = get_table_by_d66_swap(table)

    return tableName, result, number
  end

  def getWKTableResult()
    tableName = "일주일간표"
    table = [
             [11, '집에서 쫓겨난 헬P가 묶으러 왔다'],
             [12, '위험한 오오라를 낼 수 있게 되어, 엄청 인기생겼다'],
             [13, '지옥입거를 각오하고 신변정리를 했다'],
             [14, '지옥TV정보잡지에서 인터뷰를 받았다'],
             [15, '친가의 사업이 잘되고 있는데 손님이 왠지 모두 헬피플'],
             [16, '신호 기다리고 있는데 업무중인 악마가 사인요청 했다'],
             [22, '친척이 프로그램을 보고 있고 전화로 감상을 말했다'],
             [23, '다음 주의 수호천사와 아는 사이가 되버렸다'],
             [24, '다음 주의 타겟과 아는 사이가 되버렸다'],
             [25, '트라우마가 남아서 수면을 못하고 술에 빠졌다'],
             [26, '소원이 이루어지면 일 안해도 되니까,, 일 그만뒀다'],
             [33, '소원과는 별도로 출연보수가 들어와서 호와롭게 놀았다'],
             [34, '프로그램 중에 장난으로 사람을 쳤는데, 왠지 죽어버렸다'],
             [35, '프로그램 중에 악마가 무서워져서 온세상을 도망쳐다녔다'],
             [36, '팬을 자칭하는 스토커가 집을 망쳐놓았다'],
             [44, '문득 생각난 김에 산에 들어가서 수련했다'],
             [45, '헬 스타일리스트한테 집의 의류를 모조리 태워져버렸다'],
             [46, '헬 닥터가 밀린 치료비를 요구했다'],
             [55, '헬 드레스의 짝퉁을 강매하려고 한 하급 악마와 싸웠다'],
             [56, '다른 회수인 중 한명이 같은 직장인이라는 것이 확실해졌다'],
             [66, '묘하게 기부를 모집하는 영업전화가 걸려오는 주였다'],
            ]
    result, number = get_table_by_d66_swap(table)

    return tableName, result, number
  end

  def getServiceSceneTableResult()
    tableName = "서비스 씬 표"
    sceneTable = [
                  '탈의계 서비스 신표',
                  '폭력계 서비스 신표',
                  '여행계 서비스 신표',
                  '연애계 서비스 신표',
                  '감동계 서비스 신표',
                  '다른 장르계 서비스 신표',
                 ]
    sceneGroup, number1 = get_table_by_1d6(sceneTable)

    case sceneGroup
    when "탈의계 서비스 신표"
      table = [
               '의장의 문제',
               '갈아입기',
               '탈의마작',
               '입욕',
               '수영복으로 싸운다',
               '야구권',
              ]
    when "폭력계 서비스 신표"
      table = [
               '이긴 후의 폭력',
               '집단의 폭력',
               '폭력을 암시하는 대화',
               '진 후의 폭력',
               '전혀 무의미한 폭력',
               '무의미한 폭력',
              ]
    when "여행계 서비스 신표"
      table = [
               '이동기관의 소개',
               '지역의 놀이를 소개',
               '지역의 동물을 소개',
               '지역의 여관을 소개',
               '일어나면 깜짝이야',
               '명물 요리의 소개',
              ]
    when "연애계 서비스 신표"
      table = [
               '키스신',
               '사랑을 깨닫다',
               '사랑에 고민함',
               '데이트신',
               '비밀을 고백한다',
               '러브신',
              ]
    when "감동계 서비스 신표"
      table = [
               '가족의 사랑을 깨닫다',
               '멋진 걸 말한다',
               '실패해서 실의에 빠진다',
               '지역 주민과 교류한다',
               '죽은 사람 얘기를 한다',
               '높은 곳에서 멋진 걸 말한다',
              ]
    else
      table = [#"다른 장르계 서비스 신표"
               '사이코 서스펜스',
               '스페이스 오페라',
               '동화',
               '히로익 판타지',
               '몬스터 호러',
               '로보',
              ]
    end

    resultTmp, number2 = get_table_by_1d6(table)
    result = "#{sceneGroup}「#{resultTmp}」을/를 행한다."
    number = "#{number1}#{number2}"

    return tableName, result, number
  end

  def getSoulTableResult()
    tableName = "소울방출표"
    table = [
             '100',
             '200',
             '200',
             '200',
             '300',
             '300',
            ]
    result, number = get_table_by_1d6(table)

    return tableName, result, number
  end

  def getSceneTelopGenericTableResult()
    tableName = "범용 연출표"
    table = [
             '크고 알기 쉬운 자막이 나온다.',
             '회상에 맞춰 재현 비디오가 흐른다.',
             '관계자가 등장인물에 대해서 이야기한다.',
             '게스트 헬 피플이 행동을 논평한다.',
             '행동의 시비에 대해 헬 시청자의 투표가 시작된다.',
             '헬 밴드의 신곡이 BGM으로 흐른다.',
            ]
    result, number = get_table_by_1d6(table)

    return tableName, result, number
  end

  def getHairStylistAbuseTableResult(type)
    tableName = "헬 스타일리스트 매도 표"

    hellStylistAbuseTable1 = [
                              [11, ' 땀범벅이 된'],
                              [12, ' 영감이 저번에 버린'],
                              [13, ' 실패작인'],
                              [14, ' 저예산의 영화로 몬스터 대신이 된'],
                              [15, ' 무리를 잃은'],
                              [16, ' 나쁜 의미로 성분 무조정인'],
                              [22, ' 우주인이 내용물을 뺀 후의'],
                              [23, ' 화학처리반이 처리하러 올 것 같은'],
                              [24, ' 혁명 직후 단두대에 걸린'],
                              [25, ' 광기의 예술가가 사상으로 만든'],
                              [26, ' 고향 마을을 제국군에게 불태워진 직후의'],
                              [33, ' 오년 만에 침대 아래에서 나온'],
                              [34, ' 고속도로에서 마구 치인'],
                              [35, ' 사후 6주 동안의'],
                              [36, ' 실패요리를 억지로 먹인'],
                              [44, ' 갓난아기가 처음으로 그린'],
                              [45, ' 장롱과 벽 사이의 틈에서 사는'],
                              [46, ' 고양이가 물어간'],
                              [55, ' 유명한 문어를 닮은 몬스터를 본'],
                              [56, ' 여행지에서 지갑이 없단 것을 깨달은'],
                              [66, ' 엽기살인을 취미로 하고 있는'],
                             ]
    hellStylistAbuseTable2 = [
                              [11, ' 아르마딜로'],
                              [12, ' 말미잘'],
                              [13, ' 노린재'],
                              [14, ' 오리너구리'],
                              [15, ' 공작'],
                              [16, ' 해파리'],
                              [22, ' 바퀴벌레'],
                              [23, ' 벌레'],
                              [24, ' 원숭이'],
                              [25, ' 시궁쥐'],
                              [26, ' 벌거숭이'],
                              [33, ' 해삼'],
                              [34, ' 똥'],
                              [35, ' 쇠똥구리'],
                              [36, ' 펭귄'],
                              [44, ' 물벼룩'],
                              [45, ' 낡은 인형'],
                              [46, ' 식인 곰'],
                              [55, ' 독 있는 도마뱀'],
                              [56, ' 송충이'],
                              [66, ' 들개'],
                             ]

      hellStylistwtable1 = [
               '「마치『',
               '「마치『',
               '「정말로『',
               '「정말로『',
               '「너는『',
               '「너는『',
              ]
      hellStylistwtable2 = [
               '』같아!」',
               '』같구나!」',
               '』잖아!」',
               '』구나.」',
               '』처럼 보여.」',
               '』나 다를 바 없어! 」',
              ]

    case type
      when 1
        result, number = get_table_by_d66_swap(hellStylistAbuseTable1)
      when 2
        result, number = get_table_by_d66_swap(hellStylistAbuseTable2)
      else
        result1, num1 = get_table_by_d66_swap(hellStylistAbuseTable1)
        result2, num2 = get_table_by_d66_swap(hellStylistAbuseTable2)
        before,  = get_table_by_1d6(hellStylistwtable1)
        after, = get_table_by_1d6(hellStylistwtable2)
        result = "#{before}#{result1}#{result2}#{after}"
        number = "#{num1},#{num2}"
    end

    return tableName, result, number
  end

  def getSkillTableResult(type)
    skillTableFull = [
                      ['직업', ['무직', '예술가', '연구자', '가사 도우미', '학생', '악당', '노동자', '탐정', '거물', '의사', '공무원']],
                      ['동작', ['외치다', '번뜩이다', '베다', '휘두르다', '던지다', '때리다', '차다', '뛰다', '쏘다', '잡다', '기다리다']],
                      ['소도구', ['피어스', '머리장식', '총', '목걸이', '벨트', '안경', '모자', '시계', '검', '링', '타투']],
                      ['의상', ['네이킷드', '아웃도어', '에스닉', '힙합', '밀리터리', '포멀', '전통', '고딕', '펑크', '메탈', '아이돌']],
                      ['정동', ['사랑', '기쁨', '기대', '초조', '자부', '분노', '슬픔', '질투', '공포', '수치', '혐오']],
                      ['소원', ['죽음', '복수', '승리', '지배', '획득', '번영', '강화', '안전', '건강', '장수', '생']],
                     ]
    skillTable, num1 = get_table_by_1d6(skillTableFull)
    skillGroup, table = skillTable
    if(type == "T")
      tableName = "지정특기 랜덤결정표"
      skill, num2 = get_table_by_2d6(table)
      result = "「#{skillGroup}」《#{skill}》"
      number = "#{num1},#{num2}"
    else
      tableName = "지정특기분야 랜덤결정표"
      result = skillGroup
      number = num1
    end

    return tableName, result, number
  end

  def getExtraTableResult(type)
    tableName = "엑스트라 표"
    extraTable1 = [
				   [11, "당신과 친구인"],
				   [12, "당신과 원망하는 녀석의 기회를 노리고 있는"],
				   [13, "바쁘게 전화로 이야기 하고있는"],
				   [14, "넘어지며 도망친"],
				   [15, "씬의 배경의 소유자인"],
				   [16, "가공의 인물이라고 생각되고 있던 신(神)"],
				   [22, "과거의 시즌 우승자인"],
				   [23, "기분 좋게 엄청 취한"],
				   [24, "공포로 몸을 움츠리게 하고 있는"],
				   [25, "업계에서는 유명한"],
				   [26, "행복한 가정을 가지고 있는"],
				   [33, "광역 지명 수배되고 있는"],
				   [34, "오늘 모닝뉴스에서 특집으로 되어 있던"],
				   [35, "항상 미소를 잊지 않는 거물인"],
				   [36, "진실의 탐구 과정에서 발광한"],
				   [44, "현지에서는 패배를 모르는"],
				   [45, "매우 동작에 절도가 있는"],
				   [46, "평범한"],
				   [55, "변신 히어로의 정체인"],
				   [56, "『#{getNameTableResult(0)[1]}』의 이명을 가지고 있는"],
				   [66, "역사상의 인물이지만 실은 살아있던"],
	              ]
	extraTable2 = [
				   [11, "샐러리맨"],
				   [12, "스포츠 선수"],
				   [13, "양아치"],
				   [14, "드라이버"],
				   [15, "쥐"],
				   [16, "파일럿"],
				   [22, "영화 감독"],
				   [23, "개"],
				   [24, "자객"],
				   [25, "주부"],
				   [26, "소설가"],
				   [33, "작사"],
				   [34, "정치가"],
				   [35, "갑부"],
				   [36, "큰 남자"],
				   [44, "수수께끼의 미녀"],
				   [45, "고양이"],
				   [46, "미소녀"],
				   [55, "문화인"],
				   [56, "용자"],
				   [66, "신(神)"],
	              ]
    extraTable3 = [
				   [11, "이상한 상자안에서"],
				   [12, "불쌍한 희생자를 죽이면서"],
				   [13, "소를 타면서"],
				   [14, "말을 타면서"],
				   [15, "벽을 분쇄하면서"],
				   [16, "문을 분쇄하면서"],
				   [22, "진한 안개속에서 천천히"],
				   [23, "자동문에서 나오면서"],
				   [24, "상공에서 급강하하면서"],
				   [25, "전속력으로 달리면서"],
				   [26, "높은 곳으로부터"],
				   [33, "테마 음악과 함께"],
				   [34, "지나가는 길에"],
				   [35, "이름과 직업의 명함과 함께"],
				   [36, "오토바이를 타면서"],
				   [44, "무덤을 뚫으며"],
				   [45, "부하를 여럿 거느리며"],
				   [46, "거친 바람에 감기며"],
				   [55, "무기를 잡으며"],
				   [56, "헬 포탈을 지나며"],
				   [66, "불길을 등에 지며"],
	              ]
	extraTable4 = [
				   [11, "사랑스럽게 등장"],
				   [12, "선명하게 등장"],
				   [13, "시원스럽게 등장"],
				   [14, "이상하게 등장"],
				   [15, "몹시 거칠게 등장"],
				   [16, "기세 좋게 등장"],
				   [22, "아름답게 등장"],
				   [23, "잘난듯이 등장"],
				   [24, "엄숙하게 등장"],
				   [25, "무섭게 등장"],
				   [26, "멋지게 등장"],
				   [33, "뽐내며 등장"],
				   [34, "죽음의 예감과 함께 등장"],
				   [35, "조용하게 등장"],
				   [36, "품위있게 등장"],
				   [44, "야무지지 못하게 등장"],
				   [45, "지적이게 등장"],
				   [46, "나고야풍으로 등장"],
				   [55, "매끈하게 등장"],
				   [56, "기분이 안좋게 등장"],
				   [66, "쾌활하게 등장"],
	              ]

    case type
    when 1
      result, number = get_table_by_d66_swap(extraTable1)
    when 2
      result, number = get_table_by_d66_swap(extraTable2)
    when 3
      result, number = get_table_by_d66_swap(extraTable3)
    when 4
      result, number = get_table_by_d66_swap(extraTable4)
    else
      result1, num1 = get_table_by_d66_swap(extraTable1)
      result2, num2 = get_table_by_d66_swap(extraTable2)
      result3, num3 = get_table_by_d66_swap(extraTable3)
      result4, num4 = get_table_by_d66_swap(extraTable4)
      result = "#{result1}#{result2}が#{result3}#{result4}"
      number = "#{num1},#{num2},#{num3},#{num4}"
    end

    return tableName, result, number
  end

  def getpcTableResult()
    tableName = "제작위원 결정표"
    table = [
             [11, '헬 라이온'],
             [12, '헬 라이온'],
             [13, '헬 라이온'],
             [14, '헬 크로우'],
             [15, '헬 크로우'],
             [16, '헬 크로우'],
             [22, '헬 스네이크'],
             [23, '헬 스네이크'],
             [24, '헬 스네이크'],
             [25, '헬 드래곤'],
             [26, '헬 드래곤'],
             [33, '헬 드래곤'],
             [34, '헬 플라이'],
             [35, '헬 플라이'],
             [36, '헬 플라이'],
             [44, '헬 갓'],
             [45, '헬 갓'],
             [46, '헬 갓'],
             [55, '헬 베어'],
             [56, '헬 베어'],
             [66, '헬 베어'],
            ]
    result, number = get_table_by_d66_swap(table)

    return tableName, result, number
  end

  def getohTableResult()
    tableName = "실제로 어떠했는가 표"
    table = [
             '대개【흥행수입】의 예측은 정확했다. 증감은 없이, 회수인은 당분간 공공장소에서 인기였다.',
             '생각보다 흥행했다!【흥행수입】이 4D6억 헬엔 증가. 장르의 수작으로 5년 후에도 가끔 입에 오른다',
             '조금 트러블이 생겨서 공개 시기를 놓쳤다. 【흥행수입】이 4D6억 헬엔 감소. 헬-레이의 매상을 기대하자',
             '이런 맙소사, 동시기에 타시즌의 영화판이 공개된다. 【흥행수입】이 반이 된다.',
             '오 맙소사, 헬 아카데미상을 잔뜩 받았다.【흥행수입】이 두배가 된다!',
             '에피소드의 표현중에 미묘하고 귀찮은 문제를 일으켜 많은 헬 피플이 언짢게 본다. 공개는 없음으로.',
            ]
    result, number = get_table_by_1d6(table)

    return tableName, result, number
  end

  def getplTableResult()
    tableName = "헬 라이온터스크 표"
    table = [
             '
			 「프랑스어.」지정 특기는《예술가》. 왠지 등장 인물 전원이 프랑스어로 밖에 회화할 수 없는 씬이 설정된다. 할 수 있는 한, 프랑스어를 구사하고 의사소통을 시도해라!',
             '
			 「회화가 없다.」지정 특기는《뛰다》. 회화 그 자체가 비열하고 엔터테인먼트적이라고 하는 것으로, 모든 말을 발언하는 것이 허락되지 않는 씬표가 설정된다. 제스처로 힘내라!',
             '
			 「명언의 인용.」지정 특기는《머리장식》. 과거의 명작 영화나 소설, 역사상의 사건등에서 따라한 명언 밖에 말할 수 없는 씬이 설정된다. 교양이 없으면 말할 수 없다!',
             '
			 「등장인물의증가。」지정 특기는《트레드》. 등장 인물이 쓸데 없이 두배로 늘어나서 각각 개성적인 고민이나 사건을 말하기 시작한다. 이대로는 귀찮고 개런티도 늘어나므로, 전부 죽이자.',
             '
			 「막연한 불안.」지정 특기는《자부》. 특별히 무언가 정해지지않고, 멍하니  불안감에 연연 괴로워하거나 침체하거나 하는 씬을 강요 당한다. 이것을 지루하게 하지 않는 것은 어렵다!',

			 '「길게 끌기.」지정 특기는《삶》. 컷 없이 30분 정도 같은 씬이 계속 된다. NG는 용서되지 않는다. 이것으로 중간을 가지게할 수 있는 것은 프로의 배우정도의 것이지만, 회수인은 연기의 아마추어다.',
            ]
    result, number = get_table_by_1d6(table)

    return tableName, result, number
  end
  def getprTableResult()
    tableName = "헬 크로우터스크 표"
    table = [

             '「초라한 게스트.」지정 특기는《악당》. 일반인이나 개, 남동생이나 엄마가 게스트로서 등장한다. 본래의 게스트와의 차액은 착복된다. 사정을 알려져 있지 않은 게스트와 잘 관련되자!',

			 '「친가에서의 촬영.」지정 특기는《외치다》. 촬영 장소가 친가가 된다. 본래의 로케 비용은 착복된다. 가족을 설득하고, 잘 촬영 허가를 받지 않으면 적자가 나와 버릴 것이다!',

             '「평상복으로의 촬영.」지정 특기는《안경》. 왠지 헬 스타일리스트가 휴일로, 그 만큼의 인건비와 의상대로써 착복된다. 평상복인 채로 초상의 액션을 해내는 것은 불가능하게 가깝다!',

             '「도보로 이동한다.」지정 특기는《고딕》. 헬 포털의 예산이 착복되었다. 오로지 씬까지 걷지 않으면 촬영조차 할 수 없다. 촬영전에 녹초가되어 버리면, 관객에게도 전해질 것이다.',

             '「특수 효과가 나오지 않는다.」지정 특기는《기대》. 리얼타임 특수 효과의 예산이 착복되었다! 예산 없이 일하려고 하지 않는 헬 SFX 엔지니어를, 어떻게든 애원이라도 해서 일하게 하자!',

             '「식사가 나오지 않는다.」지정 특기는《획득》. 식사의 시간이지만, 헬 주문 도시락이 오지 않는다. 그 만큼의 예산이 착복되었던 것이다. 공복인 상태로 좋은 연기를 하려면 , 기합에 의지할 수 밖에 없다.',
            ]
    result, number = get_table_by_1d6(table)

    return tableName, result, number
  end
  def getpnTableResult()
    tableName = "헬 스네이크터스크 표"
    table = [

             '「원수.」지정 특기는《가사도우미》. 과거 개변에 의해, 임의의 상대가, 갑자기 부모의 원수나 연인의 원수가 된다. 당황하지 않고 분노에 불타자. 왜 이렇게 되었는지, 생각해선 안돼!',

             '「사소한 일로부터 대싸움.」지정 특기는《기다리다》. 임의의 상대와 아무래도 좋은 이유로 싸움을 하는 것을 강요 당한다. 정말로 아무래도 좋아지고 싸움을 그만두는 것은 용서되지 않는다.',

             '「이익이 상반된다.」지정 특기는《네크리스》. 임의의 상대가 바라는 것과 자신이 바라는 것이 양립하지 않는 것을 어필 당한다. 그렇지 않더라 하여도 억지로 이유를 생각해 내라!',

             '「신적.」지정 특기는《네이키드》. 취미나 종교를 이유로, 임의의 상대와 싸움 하는 것을 강요 당한다. 이야기하고 알아 버리거나 취미의 폭이 넓어져선 안 된다.',

             '「유아기의 트라우마.」지정 특기는《질투》. 과거 개변으로 슬픈 기억이 날조 되어 엽기 살인범으로서 캐릭터가 추가된다. 약간 깬다거나 슬퍼하지마라. 미쳐라!',

             '「인류 일반상식의 미움.」지정 특기는《복수》. 이유 없게 사람을 미워하는 것을 강요 당한다. 원래 성격 좋은 회수인에 있어서는 상당한 난행이다. 인류의 나쁜 면 찾기를 하자!',
            ]
    result, number = get_table_by_1d6(table)

    return tableName, result, number
  end
  def getpdTableResult()
    tableName = "헬 드래곤터스크 표"
    table = [

             ' 「바리에이션을 보고 싶어서 NG.」지정 특기는《대물》. 별로 나쁜 연기는 아닌데, 조금 다른 바리에이션을 몇번이나 할거야. 이성을 잃지 말고 참아 줘!',

             '「뭔가 다르니까 NG.」지정 특기는《때리다》. 회수인의 연기에 트집을 잡을 수 있었다. 그러나 어떤 연기를 하면 좋은 것인지 가르쳐 주지 않는다. 이성을 잃지 않고 참아 줘! 다른 사람을 때려라!',

             '「 아직 시간이 있으니까 NG.」지정 특기는《타투》. 바보같은 이유로 NG가 나왔지만, 어쨌든 이성을 잃지 않고 참아 줘. 헬 드래곤의 기분을 해친다면 광고비가 깎아진다!',

             '「예산을 다 사용하고 싶어서 NG.」지정 특기는《네이키드》. 아직 인건비를 사용할 수 있기 때문에 다시 한번, 이라든가 하는 이유로 재시도다. 이성을 잃지 않고 참아 줘. 녀석은 돈을 가지고 있어!',

             '「기분이 나쁘기 때문에 NG.」지정 특기는《화내다》. 집에서 무엇인가 싫은 일이 있었으므로 라는 둥 트집을 잡혔다. 이제, 무엇을 해도 OK는 나오지 않는다. 이성을 잃지 않고 참아 줘. 언젠가는 끝난다!',

             '「네 존재자체가 NG.」지정 특기는《안전》. 회수인의 인선 그 자체에 트집을 잡을 수 있었다. 당신에게 NG를 내는 것만이 목적이다. 이성을 잃지 않고 참아 줘. 상대는 무진장 강하다.',
            ]
    result, number = get_table_by_1d6(table)

    return tableName, result, number
  end
  def getpfTableResult()
    tableName = "헬 플라잉터스크 표"
    table = [

             '「더러운 화장실.」지정 특기는《탐정》. 몹시 비위생적인 장소로 배경이 변경된다. 똥투성이 푸세식인 화장실인지, 똥구덩이 냄새가 풀풀난다. 악취에 견딜 수 있을지, 그것이 승부의 갈림길이다!',

             '「예술적 음부 은폐.」지정 특기는《뛰다》. 전라로 씬을 내지만 비쳐선 안 되는 것 이다. 본래라면 수영복의 천이 있는 부분이 카메라에 비치지 않게, 장애물을 활용해라!',

             '「촉수.」지정 특기는《벨트》. 당돌하게 촉수 몬스터가 등장한다. 그는 신사적이지만 가차 없이 습격해오므로, 섹시하게 져라! 아니면 관객은 실망할 것이다!',
             '「미끌미끌.」지정 특기는《트레드》. 어쩐지 점액이 들어간 물통이 등장한다. 이것을 감싸고 씬을 시작하라고 말한다. 성분도 모르는 점액은 싫지만, 어떻게든 참아라!',

             '「병원체를 거쳐 입으로 섭취한다.」지정 특기는《공포》. 병의 원인이 될 것 같은 것을 억지로 먹혀진다. 구체적으로는 배설물이나 부패한 것, 처음부터 비위생적인 것 이다.',

             '「국부에의 심한 타격.」지정 특기는《죽음》. 헬 AD나 다른 캐릭터로부터 국부가 강타되는 것을 강요 당한다. 죽지 않고 잘 아파해라. 잘못하면 쇼크로 그대로 죽는다!',
            ]
    result, number = get_table_by_1d6(table)

    return tableName, result, number
  end
  def getpgTableResult()
    tableName = "헬 갓터스크 표"
    table = [

             '「아이돌이 동료로서 등장.」지정 특기는《의사》. 푸시받는 아이돌이 회수인의 아군으로서 등장하므로, 부드럽게 관계를 쌓아 올려 공투 해, 상대를 세우자. 리스펙트다!',

             '「아이돌이 적으로서 등장.」지정 특기는《차다》. 푸시받는 아이돌이 표적의 아군. 회수인의 적으로서 등장! 볼만한 장면을 준비 해 주면서, 잘 넘어뜨려라. 일방적으로는 안돼!',

             '「아이돌이 무의미하게 등장.」지정 특기는《검》. 푸시받는 아이돌이 등장하지만, 왜 등장하는지 아무도 모른다. 본인도 대본을 받지 않았다. 즉흥으로 어떻게든 해라!',

             '「아이돌의 동료가 된다.」지정 특기는《에스닉》. 푸시받는 아이돌의 동료가 되어, 당돌하게 사랑이나 모험을 한다. 황금의 영혼과 관계없는 이야기가 계속 되지만, 관객에게 그것을 눈치채게 하지 말아라!',

             '「아이돌과 악수.」지정 특기는《사랑》. 푸시받는 아이돌이 갑자기 나와 악수해 준다. 갑작스러운 일이지만, 당황해선 안 된다. 흐르는 듯이 화제를 만들며 회화해, 악수에 대해 코멘트를 하자!',

             '「같이 댄스.」지정 특기는《건강》. 갑자기 댄스 씬이 시작된다! 다만 회수인은 백 댄서다. 노래해라! 다 춤추어라! 헐떡거리거나 웃는 얼굴이 망가지면 NG다!',
            ]
    result, number = get_table_by_1d6(table)

    return tableName, result, number
  end
  def getpbTableResult()
    tableName = "헬 베어터스크 표"
    table = [
             '「헬 이불.」지정 특기는《대물》. 지옥과 같은 잘 때의 기분을 제공하는, 한 장 200만 헬엔의 헬 이불을 어필하는 것에. 소재는 절대 그렇게 좋지 않기 때문에, 기술을 강조하자.',

             '「헬 핸드백.」지정 특기는《잡다》. 무한하게 물건이 들어가는 핸드백을 어필이다. 다만 원하는 것을 꺼낼 수 없고, 가방안에 가방을 넣으면, 심한 일이 일어난다.',

             '「헬 마스크.」지정 특기는《모자》. 완전하게 헬 P의 머리 부분을 재현한 고급 마스크를 써, 그 훌륭함을 어필하면서 영화를 촬영하자. 덧붙여 시야는 몹시 나쁘다.',

             '「헬 카메라.」지정 특기는《아이돌》. 반드시 심령 사진이 찍히는 카메라를 사용하며 손님에게 강매해라! 영령사진이 나오기도 하지만, 그것을 눈치채여선 안 된다. 사진을 분명히 잘 보인다!',

             '「헬 와치.」지정 특기는《초조》. 시간은 모르지만 엄청 멋진 시계를 어필. 그 거 팔찌아냐, 라고 태클받기기 전에 잘 회유하자!',

             '「헬 부적.」지정 특기는《번영》. 소원을 성취하는 악령이, 당신의 성공을 확약! 그 다음은 성공한 채로 사고로 죽게 되지만, 그런 것 사소한 일이다!',
            ]
    result, number = get_table_by_1d6(table)

    return tableName, result, number
  end
end
